#########
# 6/5/06 note: you can now have multiple variables for RowSideColors and
# ColSideColors (the first variable for RowSideColors will be on the left and
# the first variable for ColSideColors will be on the bottom)
# To have multiple variables, use rbind for RowSideColors and cbind for
# ColSideColors.
#########
# added font and textColor as parameters!!
newheatmap<-function (x, Rowv = NULL, Colv = if (symm) "Rowv" else NULL,
    distfun = dist, hclustfun = hclust, reorderfun = function(d,
        w) reorder(d, w), add.expr, symm = FALSE, revC = identical(Colv,
        "Rowv"), scale = c("row", "column", "none"), na.rm = TRUE,
    margins = c(5, 5), ColSideColors, RowSideColors, cexRow = 0.2 +
        1/log10(nr), cexCol = 0.2 + 1/log10(nc), labRow = NULL,
    labCol = NULL, main = NULL, xlab = NULL, ylab = NULL, keep.dendro = FALSE,
    verbose = getOption("verbose"), 
    font=par("font"), textColor=par("col"), ...)
{
    # this is the return list for plots
    retPlots<-list()
    # this is the return list for par
    retPlotPar<-list()

    scale <- if (symm && missing(scale))
        "none"
    else match.arg(scale)
    if (length(di <- dim(x)) != 2 || !is.numeric(x))
        stop("'x' must be a numeric matrix")
    nr <- di[1]
    nc <- di[2]
    if (nr <= 1 || nc <= 1)
        stop("'x' must have at least 2 rows and 2 columns")
    if (!is.numeric(margins) || length(margins) != 2)
        stop("'margins' must be a numeric vector of length 2")
    doRdend <- !identical(Rowv, NA)
    doCdend <- !identical(Colv, NA)
    if (is.null(Rowv))
        Rowv <- rowMeans(x, na.rm = na.rm)
    if (is.null(Colv))
        Colv <- colMeans(x, na.rm = na.rm)
    if (doRdend) {
        if (inherits(Rowv, "dendrogram"))
            ddr <- Rowv
        else {
            hcr <- hclustfun(distfun(x))
            ddr <- as.dendrogram(hcr)
            if (!is.logical(Rowv) || Rowv)
                ddr <- reorderfun(ddr, Rowv)
        }
        if (nr != length(rowInd <- order.dendrogram(ddr)))
            stop("row dendrogram ordering gave index of wrong length")
    }
    else rowInd <- 1:nr
    if (doCdend) {
        if (inherits(Colv, "dendrogram"))
            ddc <- Colv
        else if (identical(Colv, "Rowv")) {
            if (nr != nc)
                stop("Colv = \"Rowv\" but nrow(x) != ncol(x)")
            ddc <- ddr
        }
        else {
            hcc <- hclustfun(distfun(if (symm)
                x
            else t(x)))
            ddc <- as.dendrogram(hcc)
            if (!is.logical(Colv) || Colv)
                ddc <- reorderfun(ddc, Colv)
        }
        if (nc != length(colInd <- order.dendrogram(ddc)))
            stop("column dendrogram ordering gave index of wrong length")
    }
    else colInd <- 1:nc
    x <- x[rowInd, colInd]
    labRow <- if (is.null(labRow))
        if (is.null(rownames(x)))
            (1:nr)[rowInd]
        else rownames(x)
    else labRow[rowInd]
    labCol <- if (is.null(labCol))
        if (is.null(colnames(x)))
            (1:nc)[colInd]
        else colnames(x)
    else labCol[colInd]
    if (scale == "row") {
        x <- sweep(x, 1, rowMeans(x, na.rm = na.rm))
        sx <- apply(x, 1, sd, na.rm = na.rm)
        x <- sweep(x, 1, sx, "/")
    }
    else if (scale == "column") {
        x <- sweep(x, 2, colMeans(x, na.rm = na.rm))
        sx <- apply(x, 2, sd, na.rm = na.rm)
        x <- sweep(x, 2, sx, "/")
    }
    lmat <- rbind(c(NA, 3), 2:1)
    lwid <- c(if (doRdend) 1 else 0.05, 4)
    lhei <- c((if (doCdend) 1 else 0.05) + if (!is.null(main)) 0.2 else 0,
        4)

    if (!missing(ColSideColors)) {
        if (!is.matrix(ColSideColors))
          ColSideColors<-cbind(ColSideColors)
#        if (!is.character(ColSideColors) || length(ColSideColors) != nc)
#            stop("'ColSideColors' must be a character vector of length
#                  ncol(x)")
        if (!is.character(ColSideColors) || nrow(ColSideColors) != nc)
          stop("'ColSideColors' must be a character vector of length
                ncol(x) or a matrix with the same number of rows as ncol(x)")
        lmat <- rbind(lmat[1, ] + 1, c(NA, 1), lmat[2, ] + 1)
#        lhei <- c(lhei[1], 0.2, lhei[2])
        lhei<-c(lhei[1], 0.15*ncol(ColSideColors), lhei[2])
    }
    if (!missing(RowSideColors)) {
        if (!is.matrix(RowSideColors))
          RowSideColors<-rbind(RowSideColors)
#        if (!is.character(RowSideColors) || length(RowSideColors) != nr)
#            stop("'RowSideColors' must be a character vector of length 
#                  nrow(x)")
        if (!is.character(RowSideColors) || ncol(RowSideColors) != nr)
          stop("'RowSideColors' must be a character vector of length nrow(x)
                or a matrix with the same number of columns as nrow(x)")
        lmat <- cbind(lmat[, 1] + 1, c(rep(NA, nrow(lmat) - 1),
            1), lmat[, 2] + 1)
#        lwid <- c(lwid[1], 0.2, lwid[2])
        lwid<-c(lwid[1], 0.15*nrow(RowSideColors), lwid[2])
    }
    lmat[is.na(lmat)] <- 0
    if (verbose) {
        cat("layout: widths = ", lwid, ", heights = ", lhei,
            "; lmat=\n")
        print(lmat)
    }
    ########
    # this creates the device
    ########
    op <- par(no.readonly = TRUE)
    on.exit(par(op))    
    ########
    # divides the device
    ########
    layout(lmat, widths = lwid, heights = lhei, respect = TRUE)
    if (!missing(RowSideColors)) {
        ###########
        # makes the left side heatmap
        ###########
        par(mar = c(margins[1], 0, 0, 0.5))
#        image(rbind(1:nr), col = RowSideColors[rowInd], axes = FALSE)
        nRowSideColors<-RowSideColors[, rowInd]
        image(matrix(1:length(nRowSideColors), nrow=nrow(RowSideColors)),
              col=nRowSideColors, axes=FALSE)
      
        retPlots[[length(retPlots)+1]]<-"rowHeatmap"
        retPlotPar[[length(retPlotPar)+1]]<-par(no.readonly=TRUE)
    }
    if (!missing(ColSideColors)) {
        ###########
        # makes the heatmap on top
        ###########
        par(mar = c(0.5, 0, 0, margins[2]))
#        image(cbind(1:nc), col = ColSideColors[colInd], axes = FALSE)
        nColSideColors<-ColSideColors[colInd, ]
        image(matrix(1:length(nColSideColors), ncol=ncol(ColSideColors)),
              col=nColSideColors, axes=FALSE)

        retPlots[[length(retPlots)+1]]<-"colHeatmap"
        retPlotPar[[length(retPlotPar)+1]]<-par(no.readonly=TRUE)
    }
    
    ##########
    # makes the main heatmap
    ##########
    par(mar = c(margins[1], 0, 0, margins[2]))
    
    retPlots[[length(retPlots)+1]]<-"mainHeatmap"

    if (!symm || scale != "none")
        x <- t(x)
    if (revC) {
        iy <- nr:1
        ddr <- rev(ddr)
        x <- x[, iy]
    }
    else iy <- 1:nr

    image(1:nc, 1:nr, x, xlim = 0.5 + c(0, nc), ylim = 0.5 +
        c(0, nr), axes = FALSE, xlab = "", ylab = "",...)
    
    retPlotPar[[length(retPlotPar)+1]]<-par(no.readonly=TRUE)

#    # once there are 50 or more columns or rows the text starts to overlap too
#    # much so just don't print it
#    if (nc > 50)
#      labCol<-rep("", nc)
#
#    if (nr > 50)
#      labRow<-rep("", nr)
 
    ##########
    # makes the bottom axis (one label for each column)
    ##########
    axis(1, 1:nc, labels = labCol, las = 2, line = -0.5, tick = 0,
        cex.axis = cexCol)
    ##########
    # creates the x axis label
    ##########
    if (!is.null(xlab))
        mtext(xlab, side = 1, line = margins[1] - 1.25)
    
    ##########
    # makes the right side axis (one label for each row)
    ##########
#    axis(4, iy, labels = labRow, las = 2, line = -0.5, tick = 0,
#        cex.axis = cexRow)
    # need to rearrange the colors and the fonts to match the dendrogram
    # use rowInd
    textColor<-textColor[rowInd]
    font<-font[rowInd]
    # use mtext instead of axis because will be redrawing the text labels
    mtext(text=labRow, side=4, at=iy, las=2, line=0.5,
         cex.axis=cexRow, col=textColor, font=font)

    ##########
    # creates the y axis label
    ##########
    if (!is.null(ylab))
        mtext(ylab, side = 4, line = margins[2] - 1.25)
    if (!missing(add.expr))
        eval(substitute(add.expr))
    ##########
    # plots the left side dendrogram
    ##########
    par(mar = c(margins[1], 0, 0, 0))

    if (doRdend)
    {
      plot(ddr, horiz = TRUE, axes = FALSE, yaxs = "i", leaflab = "none")
      retPlots[[length(retPlots)+1]]<-"rowDendrogram"
    }
    else frame()
    
    retPlotPar[[length(retPlotPar)+1]]<-par(no.readonly=TRUE)

    ##########
    # plots the top dendrogram
    ##########
    par(mar = c(0, 0, if (!is.null(main)) 1 else 0, margins[2]))

    if (doCdend)
    {
      plot(ddc, axes = FALSE, xaxs = "i", leaflab = "none")
      retPlots[[length(retPlots)+1]]<-"colDendrogram"
    }
    else if (!is.null(main))
        frame()

    retPlotPar[[length(retPlotPar)+1]]<-par(no.readonly=TRUE)

    ##########
    # create the main title
    ##########
    if (!is.null(main))
        title(main, cex.main = 1.5 * op[["cex.main"]])
    ##########
    # sets the return value
    # Layout is the values needed for the layout function call
    # plotPar is a list of the plotting parameters for each plot (so it's a
    #  list of lists - will have a maximum of length 5 and a minimum of length
    #  1) 
    # plotted is a list of which plots were created (will have a maximum of 
    #  length 5 and a minimum of length 1) - the possible values are: 
    #  rowHeatmap, colHeatmap, mainHeatmap (guaranteed), rowDendrogram,
    #  colDendrogram
    ##########
    invisible(list(rowInd = rowInd, colInd = colInd, Rowv = if (keep.dendro &&
        doRdend) ddr, Colv = if (keep.dendro && doCdend) ddc,
        Layout=list(width=lwid, height=lhei, layoutMat=lmat), 
        plotPar=retPlotPar, plotted=retPlots))
}

##########
# 4/29/06
# locat is the x and y location of the pointer
# dendro is the dendrogram
##########
myDendroIdentify<-function(locat, dendro)
{
#  print("in dendro identify")
  # cut the dendrogram at the height of the user click
  cutd<-cut(dendro, h=locat$y)
  # k is the number of dendrograms (from the cut)
  k<-length(cutd$lower)
  # these are the number of members in each new cluster
  clustab<-unlist(lapply(cutd$lower, function(x) {x@members}))
#  print(clustab)
  # these are the cumulative sums  
  m <- c(0, cumsum(clustab))

  xval<-locat$x
  whichCluster <- max(which(m<xval))
#  print(whichCluster)
  if (whichCluster > k)
    whichCluster<-k

  # now need to get the names of the cluster members in whichCluster
  # need to store the names in an environment because the function,
  # returnNames, uses recursion (similar to str.dendrogram)
  aMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(aMVC)
  controlEnv<-controller(curMVC)
  assign("dendNames", c(), controlEnv)
  returnNames(cutd$lower[[whichCluster]])
  return(get("dendNames", controlEnv))
}


#######
# to return the names of the elements in a dendrogram
#######
returnNames<-function(object)
{
  aMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(aMVC)
  controlEnv<-controller(curMVC)

  if (!is.leaf(object))
  {
    le<-length(object)
    for (i in 1:le)
    {
      returnNames(object[[i]])
    }
  }
  else
  {
    dendNames<-get("dendNames", controlEnv)
    dendNames<-c(dendNames, object@label)
    assign("dendNames", dendNames, controlEnv)
  }
}
