#########
# some functions from iSPlot
# and some are new
#########

######
# make an interface for getting data, in case new
# data types are added besides dataframe or matrix
######
getData<-function(modelName)
{
  curMVC<-getMVC(modelName)
  curModel<-model(curMVC)
  curData<-modelData(curModel)

  return(curData)
}

#########
# 5/24/05 create a graph view through initialize method
#
# called by initialize method for a graphView object
#########
initGraphView<-function(dataName, glayout, nShape, allowResize=TRUE)
{
  win<-gtkWindow(show=FALSE)

  if (!allowResize)
    gtkWindowSetPolicy(win, allow.shrink=FALSE, allow.grow=FALSE, 
                       auto.shrink=FALSE)
  else
    gtkWindowSetPolicy(win, allow.shrink=TRUE, allow.grow=TRUE, 
                       auto.shrink=FALSE)

  # this will create a gtk device on the window
  retList<-createGtkDev(win)
  win<-retList$win
  drArea<-retList$drArea

  curMVC<-getMVC(dataName)
  virData<-virtualData(model(curMVC))

  win$Show()

  # set up the graph layout variable using agopen
  # temporarily set the shape and width so won't get font size errors
  nodeShape<-nShape

  # the graph data
  actGraph<-getData(modelName=dataName)
  Sys.sleep(0.5)

  # for now make the labels on the nodes blank (because having trouble
  # plotting them)
  if (is.null(virData))
  {
    curlayout<-agopen(actGraph, name=dataName, layoutType=glayout,
               nodeAttrs=makeNodeAttrs(actGraph, label="", shape=nodeShape, 
                 fillcolor="transparent", color="black"))
  }
  else
  {
    # use the fillcolors and colors that are already in the virtualData slot
    # not sure if other view info should be used 
    curFillColors<-virData$fillcolor
    curColors<-virData$color
    curHide<-virData$hide
    if (any(curHide))
    {
      curFillColors[which(curHide)]<-rep("white", length(which(curHide)))
      curColors[which(curHide)]<-rep("white", length(which(curHide)))
    }

    curlayout<-agopen(actGraph, name=dataName, layoutType=glayout, 
               nodeAttrs=makeNodeAttrs(actGraph, label="", shape=nodeShape,
                 color=curColors, fillcolor=curFillColors))
  }

  newRetList<-list(win=win, drArea=drArea, plotDevice=dev.cur(),
                   plotPar=par(no.readonly=TRUE), graphLayout=curlayout)
  return(newRetList)
}

#########
# 5/24/05 create a scatterplot view through initialize method
# here just setting up the device
#
# called by initialize method for heatmapView and sPlotView
#########
initSPlotView<-function(allowResize=TRUE)
{
  win<-gtkWindow(show=FALSE)
  if (!allowResize)
    gtkWindowSetPolicy(win, allow.shrink=FALSE, allow.grow=FALSE, 
                       auto.shrink=FALSE)
  else
    gtkWindowSetPolicy(win, allow.shrink=TRUE, allow.grow=TRUE, 
                       auto.shrink=FALSE)

  # this will create a gtk device on the window
  retList<-createGtkDev(win)
  win<-retList$win
  drArea<-retList$drArea

  # make the window larger if it can't be resized (used by heatmap)
  if (!allowResize)
    gtkWidgetSetUsize(win, 650, 650)

  newRetList<-list(win=win, drArea=drArea, plotDevice=dev.cur(),
                   plotPar=par(no.readonly=TRUE))

  return(newRetList)
}

#########
# 5/24/05 create a spreadsheet view through initialize method
#
# called by initialize method for spreadView
# dfName is the name of the model (may not be a data frame)
# spreadData is the data frame to show in the spreadsheet
# colNames are the names of the columns
#########
initSpreadView<-function(dName, spreadData, colNames, allowResize=TRUE)
{
  win<-gtkWindow(show=FALSE)
  win$SetUsize(500, 400)
  if (!allowResize)
    gtkWindowSetPolicy(win, allow.shrink=FALSE, allow.grow=FALSE, 
                       auto.shrink=FALSE)
  else
    gtkWindowSetPolicy(win, allow.shrink=TRUE, allow.grow=TRUE, 
                       auto.shrink=FALSE)

  newTable<-gtkTable(1, 1, TRUE)

  # actually get the data
  showdata<-spreadData
  # need to add one to include the row names as a column
  numCols<-ncol(showdata)
  numRows<-nrow(showdata)
  # need to add a frame that shows the data
  frData<-gtkFrame(dName)     

  boxForShow<-gtkHBox(TRUE, 2)
  scShow<-gtkScrolledWindow()
  scShow$SetPolicy("automatic", "automatic")

  showDfList<-gtkCList(numCols, titles=colNames)
  showDfList$Freeze()
  for (i in 1:numRows)
  {
    text<-c(as.matrix(showdata[i,]))
    showDfList$Append(text)
  }
  showDfList$Thaw()
  # make the column titles buttons so the spreadsheet can be sorted
  gtkCListColumnTitlesActive(showDfList)
  # also want to set the column width so all items in the column
  # can be seen
  gtkCListColumnsAutosize(showDfList)
  # set the selection mode to multiple so multiple items can be selected
  # at a time (not sure if I want extended)
  gtkCListSetSelectionMode(showDfList, GtkSelectionMode[3])

  scShow$AddWithViewport(showDfList)
  boxForShow$PackStart(scShow, expand=TRUE)
  frData$Add(boxForShow)
  gtkTableAttach(newTable, frData, 0, 1, 0, 1, ypadding=10, xpadding=5)
  win$Add(newTable)
  win$Show()

  newRetList<-list(win=win, clist=showDfList)

  return(newRetList)
}

#########
# 5/24/05 add events specifically for spreadsheets
#
# called by initialize method for spreadView
#########
addEventsforSpread<-function(curView)
{
  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)
  controlEnv<-controller(curMVC)

  curclist<-clist(curView)
  gtkAddCallback(curclist, "select-row",
    function(obj, row, col, ev)
    {
      # need this variable to make sure don't end up in an infinite loop
      if (get("contLoop", controlEnv)==FALSE)
        clickEvent(curView, (row+1), event="select")
      return(TRUE)
    }
  )
   
  gtkAddCallback(curclist, "unselect-row",
    function(obj, row, col, ev)
    {
      # need this variable to make sure don't end up in an infinite loop
      if (get("contLoop", controlEnv)==FALSE)
        clickEvent(curView, (row+1), event="unselect")
      return(TRUE)
    }
  )

  # add a callback for clicking the column title buttons
  gtkAddCallback(curclist, "click_column",
    function(obj, col)
    {
      # sort based on this column
      # for now make sorting always ascending
      gtkCListSetSortType(curclist, GtkSortType[1])
      gtkCListSetSortColumn(curclist, col)
      gtkCListSort(curclist)

      # do I need to reset the view list?
      clist(curView)<-curclist
      # now add this back to the view list, the MVC, and the MVCList

    }
  ) 
}

##########
# 5/24/05 add events specifically for scatterplots
#
# called by initialize method for sPlotView
##########
addEventsforSPlots<-function(curView)
{
  gtkAddCallback(drArea(curView), "button_press_event",
    function(obj, ev)
    {
      clickEvent(curView, ev)
      return(TRUE)
    }
  )
      
  gtkAddCallback(drArea(curView), "motion-notify-event",
    function(obj, ev)
    {
      motionEvent(curView, ev)
      return(TRUE)
    }
  )  
}

############
# 5/24/05 add events specifically for graphs
#
# called by initialize method for graphView
############
addEventsforGraphs<-function(curView)
{
  gtkAddCallback(drArea(curView), "button_press_event",
    function(obj, ev)
    {
      clickEvent(curView, ev)
      return(TRUE)
    }
  )
      
  gtkAddCallback(drArea(curView), "motion-notify-event",
    function(obj, ev)
    {
      motionEvent(curView, ev)
      return(TRUE)
    }
  )  
}

############
# 4/22/06 add events specifically for heatmaps
#
# called by initialize method for heatmapView
############
addEventsforHeatmaps<-function(curView)
{
  gtkAddCallback(drArea(curView), "button_press_event",
    function(obj, ev)
    {
      clickEvent(curView, ev)
      return(TRUE)
    }
  )
}

############
# 5/24/05 add events for any type of view
# this also adds the view to the viewlist and updates
# the mvclist
#
# called by initialize method for all views
############
addEventsforViews<-function(newView)
{
  dataName<-dataName(newView)

  MVCList<-get("MVCList", mvcEnv)
  allNames<-getModelNames(sort=FALSE)
  mvcindex<-match(dataName, allNames)
  curMVC<-getMVC(dataName)
  controlEnv<-controller(curMVC)

  win<-win(newView)

  assign("activeView", newView, controlEnv)

  # need to add a callback for focus_in_event
  gtkAddCallback(win, "focus_in_event",
    function(obj, ev)
    {
      if (is(newView, "plotView"))
      {
        # set the active device
        dev.set(plotDevice(newView))
      }
      # set the active view
      assign("activeView", newView, controlEnv)
      # 6/1/06 need to store the active window number so that the view with
      # focus can be determined (this is a problem if the view doesn't have
      # a device)
      assign("activeWinNum", winNum(newView), mvcEnv)

      return(TRUE)
    }
  )

  # need a callback for focus_out_event to remove any open tooltips
  gtkAddCallback(win, "focus_out_event",
    function(obj, ev)
    {
      # no active view
      assign("activeView", c(), controlEnv)
      assign("activeWinNum", c(), mvcEnv)
   
      return(TRUE)
    }
  )

  # add the key press events for pseudo-accelerators
  createKeyPressEvent(win)        

  # add the delete event
  closeWin(newView)

  # add a new sub menu item to the control window (if it's open, otherwise 
  # add a key press event)
  winCounter<-get("winCounter", mvcEnv)
  assign("winCounter", winCounter+1, mvcEnv)

  curLabel<-paste("Window _", get("winCounter", mvcEnv),
                  "  Ctrl+", get("winCounter", mvcEnv), sep="")
  if (length(get("controlWin", mvcEnv)) > 0)
    newMenuItem<-addSubMenuItem(labelText=curLabel, action="activateWindow",
                  menuName="window", winNum=get("winCounter", mvcEnv))
  else
    addToKeyVals(text=curLabel, action="activateWindow", 
                   winNum=get("winCounter", mvcEnv))

  winNum(newView)<-get("winCounter", mvcEnv)

  # set the title of the window
  curTitle<-paste("Window ", get("winCounter", mvcEnv), sep="")
  gtkWindowSetTitle(win, curTitle)

  # add new object to viewList      
  curviewList<-viewList(curMVC)
  viewLen<-length(curviewList)
  curviewList[[viewLen+1]]<-newView
  viewList(curMVC)<-curviewList

  # need to reassign the curMVC to MVCList
  MVCList[[mvcindex]]<-curMVC
  assign("MVCList", MVCList, mvcEnv)

  return(newView)
}

#########
# 5/24/05 create plot of graph
#
# called by initialize method for graphView
#########
graphPlot<-function(curView)
{
  # set a few parameters that don't seem to be set correctly for a gtk device
  par("bg"="transparent")
  par("col"="black")
  par("fg"="black") 

  # need to get layout from the view
  curName<-dataName(curView)

  plot(curView@grLayout)

  # reset the par slot
  plotPar(curView)<-par(no.readonly=TRUE)

  return(curView)
}


#########
# added 6/29/06
# to create a qq plot (used with GSE model)
#########
makeQQplot<-function(viewItem)
{
  dName<-dataName(viewItem)
  curMVC<-getMVC(dName)
  gsTS<-gsTestStat(modelData(model(curMVC)))
  dev<-plotDevice(viewItem)

  dev.set(dev)

  curvirtualData<-virtualData(model(curMVC))
  if (length(curvirtualData) > 0)
  {
    # get the highlit column
    curHigh<-curvirtualData[, match("highlit", colnames(curvirtualData))]
    curHide<-curvirtualData[, match("hide", colnames(curvirtualData))]
    curPch<-curvirtualData[, match("pch", colnames(curvirtualData))]
    curColor<-curvirtualData[, match("color", colnames(curvirtualData))]

    # make the points that are hidden be the background color
    backgr<-"white"
    curColor[curHide]<-backgr
  }

  # it seems like the window needs to be shown before plotting or I
  # get a gdk error
  win<-win(viewItem)
  win$Show()

  # set a few parameters that don't seem to be set correctly for a gtk device
  par("bg"="transparent")
  par("col"="black")
  par("fg"="black")

  if (length(curvirtualData) > 0)
    retVal<-qqnorm(gsTS, col=curColor, pch=curPch)
  else
    retVal<-qqnorm(gsTS)

  xval(viewItem)<-retVal$x
  yval(viewItem)<-retVal$y

  if (length(curvirtualData) > 0)
  {
    # need to highlight points
    if (any(curHigh))
    {
      pointCex<-plotPar(viewItem)$cex
      curCex<-as.integer(pointCex)+1
      points(retVal$x[curHigh], retVal$y[curHigh], cex=curCex, col="red")
    }
    if (any(curHide))
    {
      backgr<-"white"
      pointCex<-plotPar(viewItem)$cex
      # this hides the point
      points(retVal$x[curHide], retVal$y[curHide], cex=pointCex, col=backgr)
    }
    # next need to check if any of these were hidden
    if (any(curHigh & curHide))
    {
      # this hides the highlighting
      points(retVal$x[curHigh & curHide], retVal$y[curHigh & curHide],
           cex=curCex, col=backgr)
    }
  }
  # need to update plot parameter usr
  plotPar(viewItem)<-par(no.readonly=TRUE)

  return(viewItem)
}

#################
# scatterplot is a function to plot a data set
# it is used so that the viewList in viewEnv can be updated
# dfName is the data set name to plot, DFrows are the rows to use
# DFcolumns are the columns to use, dev is the device number
# 
# viewItem is a sPlotView object
#
# called by initialize method for sPlotView
#################
scatterplot<-function(viewItem)
{
  dfName<-dataName(viewItem)
  # these are character so need to match
  DFrows<-dfRows(viewItem)
  DFcolumns<-c(xvar(viewItem), yvar(viewItem))  
  dev<-plotDevice(viewItem)

  # make sure the length of DFcolumns is 2
  curDF<-getData(modelName=dfName)
  rowIndeces<-match(DFrows,rownames(curDF))
  colIndeces<-match(DFcolumns,colnames(curDF))
  plotDF<-curDF[rowIndeces, colIndeces]

  dev.set(dev)

  curMVC<-getMVC(dfName)
  curvirtualData<-virtualData(model(curMVC))
  if (length(curvirtualData) > 0)
  {
    # get the highlit column
    curHigh<-curvirtualData[rowIndeces, match("highlit", names(curvirtualData))]
    curHide<-curvirtualData[rowIndeces, match("hide", names(curvirtualData))]

    # what about the data stored in the DF columns: color, highlit, pch, hide
    paramList<-setParamList(curvirtualData, rowIndeces, viewItem)  
  }

  # it seems like the window needs to be shown before plotting or I
  # get a gdk error
  win<-win(viewItem)
  win$Show()

  # set a few parameters that don't seem to be set correctly for a gtk device
  par("bg"="transparent")
  par("col"="black")
  par("fg"="black")

  # couldn't put in the parameter list directly 
  # note: may have more than one point here and this can cover up the change
  if (length(curvirtualData) > 0)
    plot(plotDF, col=paramList$col, pch=paramList$pch )
  else
    plot(plotDF)

  if (length(curvirtualData) > 0)
  {
    # need to highlight points
    highPoints<-plotDF[curHigh,]
    if (nrow(highPoints) > 0)
    {
      pointCex<-plotPar(viewItem)$cex
      curCex<-as.integer(pointCex)+1
      points(highPoints[,1], highPoints[,2], cex=curCex, 
           col="red")
    }
 
    # need to hide points
    hidePoints<-plotDF[curHide,]
    if (nrow(hidePoints) > 0)
    {
      backgr<-"white"
      # make sure to fill in the point as background in case it is colored in
      points(hidePoints[,1], hidePoints[,2], col=backgr, pch=19)
    }

    hideAndHigh<-curHigh & curHide
    if (any(hideAndHigh))
    {
      hideAndHighPoints<-plotDF[hideAndHigh,]
      points(hideAndHighPoints[,1], hideAndHighPoints[,2], cex=curCex,
            col=backgr)
    }
  }

  # need to update plot parameter usr
  plotPar(viewItem)<-par(no.readonly=TRUE)

  return(viewItem)
}

###########
# 9/1/05 create a heatmap view for an exprSet model
#
# called by initialize method for heatmapView
###########
heatmapPlot<-function(viewItem)
{
  dName<-dataName(viewItem)
  curMVC<-getMVC(dName)
  mData<-modelData(model(curMVC))
  vData<-virtualData(model(curMVC))
  controlEnv<-controller(curMVC)

  curRNames<-rNames(viewItem)
  # user curRNames to subset the model data
  if (is(mData, "exprSet"))
    mData<-mData[which(geneNames(mData) %in% curRNames), ]
  else
    mData<-mData[which(rownames(mData) %in% curRNames), ]

  vData<-vData[which(rownames(vData) %in% curRNames), ]

  # check if the annotation slot is empty
  if (mData@annotation=="" && get("chipType", controlEnv)=="")
  {
    # then check if the control window is open
    if (!is.null(get("controlWin", mvcEnv)))
    {
      # call get chip type
      getChipType()
      cType<-get("chipType", controlEnv)
      require(cType, character.only=TRUE) || stop("The ", cType, " package is required to continue")
      curSYMBENV<-paste(cType, "SYMBOL", sep="")
    } 
    else
    {
      print(paste("Please call the function, setChipType, if you would like",
                  "to see the gene symbols rather than affy ids on the",
                  "heatmap view."))
      curSYMBENV<-""
    }
  }  
  else
  {
    if (mData@annotation!="")
    {
      require(mData@annotation, character.only=TRUE) || stop("The ", mData@annotation, " package is required to continue")
      curSYMBENV<-paste(mData@annotation, "SYMBOL", sep="")
    }
    else
    {
        tmpPName <- get("chipType", controlEnv)
      require(tmpPName, character.only=TRUE) || stop("The ", tmpPName, " package is required to continue")
      curSYMBENV<-paste(get("chipType", controlEnv), "SYMBOL", sep="")
    }
  }

  dev<-plotDevice(viewItem)
  dev.set(dev)
  w<-win(viewItem)
  w$Show()
  # need to slow things down so the window is shown before it tries to
  # create the plot - same thing as happens in initGraphView
  Sys.sleep(0.5)

  # check if any genes are hidden
  curHidden<-vData$hide
  if (any(curHidden))
  {
    rowsToHide<-which(curHidden)
    # then subset the model data
    mData<-mData[-rowsToHide, ]
  }

  # then look at color and highlit
  curColor<-vData$color
  if (any(curHidden))
    curColor<-curColor[-rowsToHide]
  curHighlit<-vData$highlit
  curFont<-rep(1, nrow(vData))
  curFont[curHighlit]<-4
  if (any(curHidden))
    curFont<-curFont[-rowsToHide]

  par(bg="transparent")
  par(col="black")
  par(fg="black")

  # now convert affy ids to symbols if we can
  if (curSYMBENV != "")
  {
    curAffyIds<-geneNames(mData)
    # some of the SYMBOLS may be NA
    curSymb<-unlist(mget(curAffyIds, eval(as.name(curSYMBENV))))
    curSymb[which(is.na(curSymb))]<-names(curSymb[which(is.na(curSymb))])
    dataToPlot<-exprs(mData)
    rownames(dataToPlot)<-curSymb
  }
  else
  {
    dataToPlot<-exprs(mData)
  }

  curlabRow<-rownames(dataToPlot)
  if (length(curlabRow) > 50)
    curlabRow[curColor=="black" & curFont==1]<-""

  curlabCol<-colnames(dataToPlot)
  if (length(curlabCol) > 50)
    curlabCol<-rep("", length(curlabCol)) 

  # now plot the heatmap
  # call the newheatmap function that is defined in the iSNetwork package
  retList<-newheatmap(dataToPlot, margins=c(7,8), textColor=curColor, 
                      font=curFont, keep.dendro=TRUE, labRow=curlabRow,
                      labCol=curlabCol)
  ordering(viewItem)<-retList

  plotPar(viewItem)<-par(no.readonly=TRUE)

  return(viewItem)
}

#################
# setParamList is a function to set the parameter list for making a 
# scatterplot
# dataF is the dataframe and dfRows are the dataframe rows
#
# called by scatterplot function
#################
setParamList<-function(dataF,dfRows,plotObject)
{
  colIndex<-match("color", names(dataF))
  curColors<-dataF[dfRows, colIndex]
  pchIndex<-match("pch", names(dataF))
  curPch<-dataF[dfRows, pchIndex]

  hideIndex<-match("hide", names(dataF))
  curHide<-dataF[dfRows, hideIndex]
  highIndex<-match("highlit", names(dataF))
  curHigh<-dataF[dfRows, highIndex]
  
  # make the points that are hidden be the background color
  plotBG<-plotPar(plotObject)$bg
  curColors[curHide]<-plotBG

  return(list(col=curColors, pch=curPch))
}

################
# createGtkDev is a function to create a Gtk device for plotting
# -also adds a callback for the focus event
#
# called by initGraphView and initSPlotView functions
################
createGtkDev<-function(w)
{
  dev <- c()

  w$SetUsize(500, 500)

  # need to add rulers
  vbox<-gtkVBox()
  hrul<-gtkHRuler(show=FALSE)
  hrul$SetMetric("pixels")
  hrul$SetRange(0, 100, 0, 10)
  gtkBoxPackStart(vbox,hrul,expand=FALSE)

  hbox<-gtkHBox()
  gtkBoxPackStart(vbox,hbox)
  vrul<-gtkVRuler(show=FALSE)
  vrul$SetMetric("pixels")
  vrul$SetRange(0, 100, 0, 10)
  gtkBoxPackStart(hbox,vrul,expand=FALSE)

  w$Add(vbox)
 
  drArea <- gtkDrawingArea()
  dev<-addDrawingArea(hbox, drArea, 250, 250, dev, devName="Dev1", 
                      addType=5)

  return(list(win=w, drArea=drArea))
}

