###########
# first the methods for the models
###########

#######
# initialize methods for models
#######

setMethod("initialize", "exprModel", 
  function(.Object, ...)
  {
    modelData<-list(...)$modelData
    modelName<-list(...)$modelName
    linkData<-list(...)$linkData
    virtualData<-list(...)$virtualData
    modelVar<-list(...)$modelVar
    if (is.null(modelVar))
      modelVar<-list()

    if (is.null(virtualData))
    {
      # need to create the virtualData slot - will be the same as the
      # virtualData slot for a dfModel
      defaultPlotData<-list(color="black", pch=1, highlit=FALSE, hide=FALSE)
      numRows<-nrow(exprs(modelData))
      if (length(defaultPlotData) > 0)
      {
        virData<-data.frame(rep(defaultPlotData[[1]], numRows), 
                            row.names=rownames(exprs(modelData)))
        virData[,1]<-as.character(virData[,1])
        colnames(virData)<-names(defaultPlotData)[1]
      }
      if (length(defaultPlotData) > 1)
      {
        for (i in 2:length(defaultPlotData))
        {
          virData[,i]<-rep(defaultPlotData[[i]], numRows)
        }
        colnames(virData)<-names(defaultPlotData)
      }
    }
    else
      virData<-virtualData

    # now assign values to the object and then return object
    .Object@modelData<-modelData
    .Object@linkData<-linkData
    .Object@virtualData<-virData
    .Object@modelName<-modelName
    .Object@modelVar<-modelVar
    .Object
  }
)

setMethod("initialize", "dfModel",
  function(.Object, ...)
  {
    modelData<-list(...)$modelData
    modelName<-list(...)$modelName
    linkData<-list(...)$linkData
    virtualData<-list(...)$virtualData
    modelVar<-list(...)$modelVar
    if (is.null(modelVar))
      modelVar<-list()

    # need to check the class of the model data
    if (is(modelData, "matrix"))
      modelData<-as.data.frame(modelData)

    if (is.null(virtualData))
    {
      # need to create the virtualData slot 
      defaultPlotData<-list(color="black", pch=1, highlit=FALSE, hide=FALSE)
      numRows<-nrow(modelData)
      if (length(defaultPlotData) > 0)
      {
        virData<-data.frame(rep(defaultPlotData[[1]], numRows), 
                            row.names=rownames(modelData))
        virData[,1]<-as.character(virData[,1])
        colnames(virData)<-names(defaultPlotData)[1]
      }
      if (length(defaultPlotData) > 1)
      {
        for (i in 2:length(defaultPlotData))
        {
          virData[,i]<-rep(defaultPlotData[[i]], numRows)
        }
        colnames(virData)<-names(defaultPlotData)
      }
    }
    else
      virData<-virtualData
    # now assign values to the object and then return object
    .Object@modelData<-modelData
    .Object@linkData<-linkData
    .Object@virtualData<-virData
    .Object@modelName<-modelName
    .Object@modelVar<-modelVar
    .Object
  }
)

setMethod("initialize", "graphModel",
  function(.Object, ...)
  {
    modelData<-list(...)$modelData
    modelName<-list(...)$modelName
    linkData<-list(...)$linkData
    virtualData<-list(...)$virtualData
    modelVar<-list(...)$modelVar
    if (is.null(modelVar))
      modelVar<-list()

    if (is.null(virtualData))
    {
      # need to create the virtualData slot 
      defaultPlotData<-list(color="black", fillcolor="white", hide=FALSE)

      numRows<-length(nodes(modelData))

      if (length(defaultPlotData) > 0)
      {
        virData<-data.frame(rep(defaultPlotData[[1]], numRows), 
                            row.names=nodes(modelData))
        virData[,1]<-as.character(virData[,1])
        colnames(virData)<-names(defaultPlotData)[1]
      }
      if (length(defaultPlotData) > 1)
      {
        for (i in 2:length(defaultPlotData))
        {
          virData[,i]<-rep(defaultPlotData[[i]], numRows)
        }
        colnames(virData)<-names(defaultPlotData)
      }
    }
    else
      virData<-virtualData

    # now assign values to the object and then return object
    .Object@modelData<-modelData
    .Object@linkData<-linkData
    .Object@virtualData<-virData
    .Object@modelName<-modelName
    .Object@modelVar<-modelVar
    .Object
  }
)

setMethod("initialize", "gseModel",
  function(.Object, ...)
  {
    modelData<-list(...)$modelData
    modelName<-list(...)$modelName
    linkData<-list(...)$linkData
    virtualData<-list(...)$virtualData
    modelVar<-list(...)$modelVar
    if (is.null(modelVar))
      modelVar<-list()

    # not sure how to set the virtual data slot yet!
    if (is.null(virtualData))
    {
      # need to create the virtualData slot 
      defaultPlotData<-list(color="black", pch=1, highlit=FALSE, hide=FALSE)
      numRows<-length(gsTestStat(modelData))

      if (length(defaultPlotData) > 0)
      {
        virData<-data.frame(rep(defaultPlotData[[1]], numRows), 
                            row.names=rownames(incidMat(modelData)))
        virData[,1]<-as.character(virData[,1])
        colnames(virData)<-names(defaultPlotData)[1]
      }
      if (length(defaultPlotData) > 1)
      {
        for (i in 2:length(defaultPlotData))
        {
          virData[,i]<-rep(defaultPlotData[[i]], numRows)
        }
        colnames(virData)<-names(defaultPlotData)
      }
    }
    else
      virData<-virtualData

    # now assign values to the object and then return object
    .Object@modelData<-modelData
    .Object@linkData<-linkData
    .Object@virtualData<-virData
    .Object@modelName<-modelName
    .Object@modelVar<-modelVar
    .Object
  }
)

############
# update model methods
############

###########
# 8/31/05 need to determine how the update model method will work for 
# an exprSet model
#
# for the virtualData slot for an exprModel is the same as for a dfModel
###########
setMethod("updateModel", "exprModel",
  function(object, type, data)
  {
    # for a dataframe want to change the virtualData slot
    virData<-virtualData(object)
    dataName<-modelName(object)

    colIndex<-match(type, colnames(virData))
    rowName<-unique(names(data))
    rowIndex<-match(rowName, rownames(virData))

    oldValue<-virData[rowIndex, colIndex]
    # update the value
    virData[rowIndex, colIndex]<-unlist(data)

    # now update the objects
    virtualData(object)<-virData

    # need to update the MVC object - may not be the active MVC
    curMVC<-getMVC(dataName)
    model(curMVC)<-object

    # will also need to update the MVCList
    mvcList <- get("MVCList", mvcEnv)
    allNames <- getModelNames(sort = FALSE)

    index<-match(dataName, allNames)
    mvcList[[index]]<-curMVC
    assign("MVCList", mvcList, mvcEnv)

    # return the data needed to update the views
    # need the old value to remove the old point from the plot before
    # re-adding the point with the new value
    viewData<-list(colName=type, rowName=rowName, oldValue=oldValue, 
                   newValue=unlist(data))
    return(viewData)
  }
)

setMethod("updateModel", "dfModel",
  function(object, type, data)
  {
    # for a dataframe want to change the virtualData slot
    virData<-virtualData(object)
    dataName<-modelName(object)

    colIndex<-match(type, colnames(virData))
    rowName<-unique(names(data))
    rowIndex<-match(rowName, rownames(virData))

    oldValue<-virData[rowIndex, colIndex]
    # update the value
    # by performing unlist - all objects will be forced to be the same class
    # here that means numeric is converted to character
#    virData[rowIndex, colIndex]<-unlist(data)
    # so if more than one column of the data frame needs to change then
    # need to be careful of using unlist
    if (length(colIndex) > 1)
    {
      curIndex<-1
      # then we are changing both pch and color
      for (i in 1:length(colIndex))
      {
        virData[rowIndex, colIndex[i]]<-unlist(data[curIndex:(curIndex+length(rowIndex)-1)])
        curIndex<-curIndex+length(rowIndex)
      }
    }
    else
      virData[rowIndex, colIndex]<-unlist(data)

    # now update the objects
    virtualData(object)<-virData

    # need to update the MVC object - may not be the active MVC
    curMVC<-getMVC(dataName)
    model(curMVC)<-object

    # will also need to update the MVCList
    mvcList <- get("MVCList", mvcEnv)
    allNames <- getModelNames(sort = FALSE)

    index<-match(dataName, allNames)
    mvcList[[index]]<-curMVC
    assign("MVCList", mvcList, mvcEnv)

    # return the data needed to update the views
    # need the old value to remove the old point from the plot before
    # re-adding the point with the new value
    viewData<-list(colName=type, rowName=rowName, oldValue=oldValue, 
                   newValue=unlist(data))
    return(viewData)
  }
)

########
# 7/29/05 update the virtualData slot in the model due to a change
# in the node values (in the future may want to update edges)
########
setMethod("updateModel", "graphModel",
  function(object, type, data)
  {
    nameType<-names(type)
    if (all(nameType=="node")) 
    {
      # need to update the virtualData slot
      curColumn<-unique(type)
      curNode<-names(data)
      curValue<-unlist(data)

      dataName<-modelName(object)
      curMVC<-getMVC(dataName)
      # need to update the virtual data slot
      virData<-virtualData(object)

      rowToChange<-match(curNode, rownames(virData))
      colToChange<-match(curColumn, colnames(virData))      

      oldValue<-virData[rowToChange, colToChange]
      virData[rowToChange, colToChange]<-curValue

      # now update the model
      virtualData(object)<-virData
      model(curMVC)<-object      

      # need to update the MVC list
      mvcList <- get("MVCList", mvcEnv)
      allModelNames <- getModelNames(sort = FALSE)
      index<-match(dataName, allModelNames)
      mvcList[[index]]<-curMVC
      assign("MVCList", mvcList, mvcEnv)

      # need to return a list - the names of the list will be node names
#      return(as.list(data))
      viewData<-list(colName=curColumn, rowName=curNode, 
                   oldValue=oldValue, newValue=curValue)
      return(viewData)
    }
    # for the future
    if (all(nameType=="edge"))
    {
    }
  }
)

setMethod("updateModel", "gseModel",
  function(object, type, data)
  { 
    # for a dataframe want to change the virtualData slot
    virData<-virtualData(object)
    dataName<-modelName(object)

    colIndex<-match(type, colnames(virData))
    rowName<-unique(names(data))
    rowIndex<-match(rowName, rownames(virData))

    oldValue<-virData[rowIndex, colIndex]
    # update the value
    # if more than one column of the data frame needs to change then
    # need to be careful of using unlist
    if (length(colIndex) > 1)
    {
      curIndex<-1
      # then we are changing both pch and color
      for (i in 1:length(colIndex))
      {
        virData[rowIndex, colIndex[i]]<-
             unlist(data[curIndex:(curIndex+length(rowIndex)-1)])
        curIndex<-curIndex+length(rowIndex)
      }
    }
    else
      virData[rowIndex, colIndex]<-unlist(data)
    # now update the objects
    virtualData(object)<-virData

    # need to update the MVC object - may not be the active MVC
    curMVC<-getMVC(dataName)
    model(curMVC)<-object

    # will also need to update the MVCList
    mvcList <- get("MVCList", mvcEnv)
    allNames <- getModelNames(sort = FALSE)

    index<-match(dataName, allNames)
    mvcList[[index]]<-curMVC
    assign("MVCList", mvcList, mvcEnv)

    # return the data needed to update the views
    # need the old value to remove the old point from the plot before
    # re-adding the point with the new value
    viewData<-list(colName=type, rowName=rowName, oldValue=oldValue, 
                   newValue=unlist(data))
    return(viewData)
  }
)

########
# 6/21/06 provideInfo method for exprModel
# will return an exprSet, which will probably be a subset of the original
# exprSet
########
setMethod("provideInfo", "exprModel",
  function(object, type, data)
  {
    modName<-modelName(object)
    modData<-modelData(object)
    # type will be gene names (could be sample names)
    # type can also be controller!!! this lets a child ask for controller 
    #   environment variables
    if (type=="genes")
    {
      curData<-data[[type]]
      # then data will be the gene names
      if (any(curData %in% geneNames(modData)))
      {
        # then return the subset exprSet
        # get the row indices
        rowIndex<-which(geneNames(modData) %in% curData)
        subsetExpr<-modData[rowIndex, ]
        return(subsetExpr)
      }
      else
        return(NULL)
    }
    if (type=="samples")
    {
      curData<-data[[type]]
      # then data will be the sample names
      if (any(curData %in% sampleNames(modData)))
      {
        # then return the subset exprSet
        # get the column indices
        colIndex<-which(sampleNames(modData) %in% curData)
        subsetExpr<-modData[, colIndex]
        return(subsetExpr)
      }
      else
        return(NULL)
    }
    if (type=="controller")
    {
      curMVC<-getMVC(modName)
      controlEnv<-controller(curMVC)
      # data will be a vector of keys in the environment
      curData<-data[[type]]
      if (any(curData %in% ls(controlEnv)))
      {
        availData<-curData[curData %in% ls(controlEnv)]
        controlVar<-mget(availData, controlEnv)
        return(controlVar)
      }
      else
        return(NULL)      
    }
  }
)

setMethod("provideInfo", "dfModel",
  function(object, type, data)
  {
    modData<-modelData(object)
    modName<-modelName(object)
    # type can be either rows or columns
    if (type=="rows")
    {
      curData<-data[[type]]
      # then data will be the gene names
      if (any(curData %in% rownames(modData)))
      {
        # then return the subset data frame
        # get the row indices
        rowIndex<-which(rownames(modData) %in% curData)
        subsetDf<-modData[rowIndex, ]
        return(subsetDf)
      }
      else
        return(NULL)
    }
    if (type=="columns")
    {
      curData<-data[[type]]
      if (any(curData %in% colnames(modData)))
      {
        # then return the subset data frame
        # get the column indices
        colIndex<-which(colnames(modData) %in% curData)
        subsetDf<-modData[, colIndex]
        return(subsetDf)
      }
      else
        return(NULL)
    }
    if (type=="controller")
    {
      curMVC<-getMVC(modName)
      controlEnv<-controller(curMVC)
      # data will be a vector of keys in the environment
      curData<-data[[type]]
      if (any(curData %in% ls(controlEnv)))
      {
        availData<-curData[curData %in% ls(controlEnv)]
        controlVar<-mget(availData, controlEnv)
        return(controlVar)
      }
      else
        return(NULL)      
    }
  }
)

setMethod("provideInfo", "graphModel",
  function(object, type, data)
  {
    modData<-modelData(object)
    modName<-modelName(object)
    # type can be either nodes or edges (for now just focus on nodes)
    if (type=="nodes")
    {
      curData<-data[[type]]
      if (any(curData %in% nodes(modData)))
      {
        # then return the subset graph
        nodeNames<-curData[curData %in% nodes(modData)]
        subsetGraph<-subGraph(nodeNames, modData)
        return(subsetGraph)
      }
      else
        return(NULL)
    }
    if (type=="edges")
    {
      # would need to think about subsetting based on edges
      return(NULL)
    }
    if (type=="controller")
    {
      curMVC<-getMVC(modName)
      controlEnv<-controller(curMVC)
      # data will be a vector of keys in the environment
      curData<-data[[type]]
      if (any(curData %in% ls(controlEnv)))
      {
        availData<-curData[curData %in% ls(controlEnv)]
        controlVar<-mget(availData, controlEnv)
        return(controlVar)
      }
      else
        return(NULL)      
    }
  }
)


###########
# update view methods
###########

setMethod("updateView", "sPlotView",
  function(object, vData)
  {
    # vData is a list with 4 elements: rowName, colName, oldValue and newValue
    # will need to remove old value and draw point with new value

    # get the model info.
    dataName<-dataName(object)
    curMVC<-getMVC(dataName)
    model<-model(curMVC)
    modelData<-modelData(model)
    virtualData<-virtualData(model)

    # set the current device
    curDevice<-plotDevice(object)
    dev.set(curDevice)

    # will need the background color
    bgColor<-plotPar(object)$bg
    if (bgColor=="transparent")
      bgColor<-"white"

    rowName<-vData$rowName
    # this will be a column in the virtualData slot
    colName<-vData$colName
    oldValue<-vData$oldValue
    newValue<-vData$newValue

    # these are columns in the modelData slot
    varx<-xvar(object)
    vary<-yvar(object)
    dfRows<-dfRows(object)
    # only update this point if it is being shown in the current plot
    if (all(rowName %in% dfRows))
    {
      # get the x and y values
      rowIndex<-match(rowName, rownames(modelData))
      xcolIndex<-match(varx, colnames(modelData))
      ycolIndex<-match(vary, colnames(modelData))
      xvalue<-modelData[rowIndex, xcolIndex]
      yvalue<-modelData[rowIndex, ycolIndex]

      pchIndex<-match("pch", colnames(virtualData))
      pchValue<-virtualData[rowIndex, pchIndex]
      colIndex<-match("color", colnames(virtualData))
      colValue<-virtualData[rowIndex, colIndex]
      highIndex<-match("highlit", colnames(virtualData))
      highValue<-virtualData[rowIndex, highIndex]
      hideIndex<-match("hide", colnames(virtualData))
      hideValue<-virtualData[rowIndex, hideIndex]

      # next remove the old point
      # what about highlighting??? - would also need to remove this!
      # need to remove old point based on the pch value
      if (any(colName == "pch"))
      {
        if (is(oldValue, "data.frame"))
        {
          curPchIndex<-match("pch", colName)
          curOldValue<-oldValue[,curPchIndex]
        }
        else
          curOldValue<-oldValue
        points(xvalue, yvalue, col=bgColor, pch=as.numeric(curOldValue))
      }
      else
        points(xvalue, yvalue, col=bgColor, pch=as.numeric(pchValue)) 

      # remove highlighting if it's needed
      if (any(highValue==TRUE) && colName!="highlit")
        points(xvalue[highValue], yvalue[highValue], col=bgColor, cex=2,
                  pch=1)
      # note that highValue and newValue will be the same here because colName
      # is highlit and the model has already been updated - only remove 
      # highlighting on those with a highValue of FALSE
      if (colName=="highlit" && any(newValue==FALSE))
        points(xvalue[!highValue], yvalue[!highValue], col=bgColor, 
                      cex=2, pch=1)

      # only redraw the point if it's not supposed to be hidden
      if (!(colName=="hide" && all(newValue==TRUE))) 
      {
        # then redraw the point - some points may be hidden
        points(xvalue[!hideValue], yvalue[!hideValue], col=colValue, 
               pch=as.numeric(pchValue))

        # add highlighting if it's needed
        if (any(highValue==TRUE & (hideValue==FALSE)))
          points(xvalue[which(highValue & !hideValue)], 
                 yvalue[which(highValue & !hideValue)], 
                      col="red", pch=1, cex=2)
      }
    }
  }
)

setMethod("updateView", "qqPlotView",
  function(object, vData)
  {
    # vData is a list with 4 elements: rowName, colName, oldValue and newValue
    # will need to remove old value and draw point with new value

    # get the model info.
    dataName<-dataName(object)
    curMVC<-getMVC(dataName)
    model<-model(curMVC)
    modelData<-modelData(model)
    virtualData<-virtualData(model)

    # set the current device
    curDevice<-plotDevice(object)
    dev.set(curDevice)

    # will need the background color
    bgColor<-plotPar(object)$bg
    if (bgColor=="transparent")
      bgColor<-"white"

    rowName<-vData$rowName
    # this will be a column in the virtualData slot
    colName<-vData$colName
    oldValue<-vData$oldValue
    newValue<-vData$newValue

    # these are columns in the modelData slot

    # get the x and y values
    rowIndex<-match(rowName, rownames(virtualData))
    xvalue<-xval(object)[rowIndex]
    yvalue<-yval(object)[rowIndex]

    pchIndex<-match("pch", colnames(virtualData))
    pchValue<-virtualData[rowIndex, pchIndex]
    colIndex<-match("color", colnames(virtualData))
    colValue<-virtualData[rowIndex, colIndex]
    highIndex<-match("highlit", colnames(virtualData))
    highValue<-virtualData[rowIndex, highIndex]
    hideIndex<-match("hide", colnames(virtualData))
    hideValue<-virtualData[rowIndex, hideIndex]

    # first remove the old point
    # what about highlighting??? - would also need to remove this!
    # need to remove old point based on the pch value
    if (any(colName == "pch"))
    {
      if (is(oldValue, "data.frame"))
      {
        curPchIndex<-match("pch", colName)
        curOldValue<-oldValue[, curPchIndex]
      }
      else
        curOldValue<-oldValue
      points(xvalue, yvalue, col=bgColor, pch=as.numeric(curOldValue))
    }
    else
      points(xvalue, yvalue, col=bgColor, pch=as.numeric(pchValue)) 

    # remove highlighting if it's needed:
    # if something other than highlighting was changed - then remove 
    # old highlighting - will be re-added if the column is not hidden
    if (any(highValue==TRUE) && colName!="highlit")
      points(xvalue[highValue], yvalue[highValue], col=bgColor, cex=2, pch=1)
    # if highlighting was changed, then remove highlighting on some
    if (colName=="highlit" && any(newValue==FALSE))
      points(xvalue[!highValue], yvalue[!highValue], col=bgColor, cex=2, pch=1)

    # only redraw the point if it's not supposed to be hidden
    if (!(colName=="hide" && all(newValue==TRUE))) 
    {
      # then redraw the points that are not supposed to be hidden
      points(xvalue[!hideValue], yvalue[!hideValue], col=colValue, 
             pch=as.numeric(pchValue))

      # add highlighting if it's needed
      if (any(highValue==TRUE & (hideValue==FALSE)))
        points(xvalue[which(highValue & !hideValue)], 
               yvalue[which(highValue & !hideValue)], 
                    col="red", pch=1, cex=2)
    }
  }
)

setMethod("updateView", "spreadView",
  function(object, vData)
  {
    # vData is a list with 4 elements: rowName, colName, oldValue and newValue

    # get the model info. - may not be from the active MVC
    curMVC<-getMVC(dataName(object))
    model<-model(curMVC)
    modelData<-modelData(model)
    virtualData<-virtualData(model)

    curCList<-clist(object)
    numCListRows<-gtkCListGetRows(curCList)
    firstRowText<-gtkCListGetText(curCList, 0:(numCListRows-1), 0)

    colName<-vData$colName
    rowName<-vData$rowName
    if (is(model, "dfModel") || is(model, "gseModel"))
    {
#      rowIndex<-match(rowName, rownames(modelData))
      rowIndex<-match(rowName, firstRowText)

      # by using ||, it will only return one value so this is ok if colName is
      # of length greater than 1
      if (colName=="color" || colName=="pch")
      {
        for (i in 1:length(rowIndex))
        {
          # always select the row when a point is being colored
          gtkCListSelectRow(curCList, rowIndex[i]-1, -1)
        }
      }
      else
      {
        # for highlight and hide need to toggle the selection
        # if value==TRUE then select, otherwise unselect
        newValue<-vData$newValue
        if (all(newValue==TRUE))
        {
          for (i in 1:length(rowIndex))
          {
            gtkCListSelectRow(curCList, rowIndex[i]-1, -1)
          }
        }
        else
        {
          for (i in 1:length(rowIndex))
          {
            gtkCListUnselectRow(curCList, rowIndex[i]-1, -1)
          }
        }
      }
    }
    if (is(model, "graphModel"))
    {
#      rowIndex<-match(rowName, nodes(modelData))
      rowIndex<-match(rowName, firstRowText)
      if (colName == "fillcolor")
      {
        for (i in 1:length(rowIndex))
        { 
          gtkCListSelectRow(curCList, rowIndex[i]-1, -1)
        }
      }
      else
      {
        if (colName == "color")
        {
          if (all(vData$newValue == "red"))
          {
            for (i in 1:length(rowIndex))
            {
              gtkCListSelectRow(curCList, rowIndex[i]-1, -1)
            }
          }
          else
          {
            for (i in 1:length(rowIndex))
            {
              gtkCListUnselectRow(curCList, rowIndex[i]-1, -1)
            }
          }
        }
        else
        {
          if (all(vData$newValue == TRUE))
          {
            for (i in 1:length(rowIndex))
            {
              gtkCListSelectRow(curCList, rowIndex[i]-1, -1)
            }
          }
          else
          {
            for (i in 1:length(rowIndex))
            {
              gtkCListUnselectRow(curCList, rowIndex[i]-1, -1)
            }
          }
        }
      }
    }
  }
)

##########
# 7/29/05 vData is the changed node
# may want to change this in the future if you want to update edges
##########
setMethod("updateView", "graphView",
  function(object, vData)
  {
    curdev<-dev.cur()
    plotdev<-plotDevice(object)
    dev.set(plotdev)

    # vData is a list of node names and the new values
    viewGrLayout<-grLayout(object)
    allNodes<-AgNode(viewGrLayout)
    allNodeNames<-unlist(lapply(allNodes, name))

#    nodesToChange<-names(vData)
    nodesToChange<-vData$rowName

    # need to get the virtualData slot of the model
    # get the model info. - may not be from the active MVC
    curMVC<-getMVC(dataName(object))
    model<-model(curMVC)
    modelData<-modelData(model)
    virtualData<-virtualData(model)    

    # now figure out which nodes need to change
    newVData<-vector(mode="list", length=length(nodesToChange))
    for (i in 1:length(nodesToChange))
    {
      curNodeName<-nodesToChange[i]
      curIndex<-match(curNodeName, allNodeNames)
      newVData[[i]]<-allNodes[[curIndex]]
      
      # now check the values in virtualData
      curVirDataIndex<-match(curNodeName, rownames(virtualData))
      newVData[[i]]@color<-virtualData[curVirDataIndex, 
                              match("color", colnames(virtualData))]
      newVData[[i]]@fillcolor<-virtualData[curVirDataIndex,
                              match("fillcolor", colnames(virtualData))]
      # now check if hide == TRUE
      if (virtualData[curVirDataIndex, match("hide", colnames(virtualData))])
      {
        newVData[[i]]@color<-"white"
        newVData[[i]]@fillcolor<-"white"
      }
    }

    # need to call drawAgNode based on the new node passed in vData
    # newVData is a list of AgNodes that may have length > 1
    x<-lapply(newVData, drawAgNode)

    dev.set(curdev)
  }
)

####
# need to create this method (not sure what I want to update in the view)
####
setMethod("updateView", "heatmapView",
  function(object, vData)
  {
    # 4/25/06 now let a user highlight a gene, color a gene and hide a gene

    # vData is a list with 4 elements: rowName, colName, oldValue and newValue

    # first need to update vData so it matches the rows that are shown in this
    # view
    # these are the rows shown in the view
    curRNames<-rNames(object)
    # these are the rows to update
    curGeneName<-vData$rowName    
    # only need to update the rows that are shown in the view
    curGeneName<-curGeneName[vData$rowName %in% curRNames]
    if (is(vData$oldValue, "data.frame"))
      oldVals<-vData$oldValue[vData$rowName %in% curRNames, ]
    else
      oldVals<-vData$oldValue[vData$rowName %in% curRNames]
    newVals<-vData$newValue[vData$rowName %in% curRNames]

    vData<-list("colName"=vData$colName, "rowName"=curGeneName,
                  "oldValue"=oldVals, "newValue"=newVals)

    if (length(curGeneName) > 0)
    {
      curdev<-dev.cur()

      # get the model info.
      dataName<-dataName(object)
      curMVC<-getMVC(dataName)
      model<-model(curMVC)
      modelData<-modelData(model)
      virtualData<-virtualData(model)
      curOrdering<-ordering(object)
      curRowInd<-curOrdering$rowInd
      curRowv<-curOrdering$Rowv
      curPlotPar<-curOrdering$plotPar
      curPlotted<-curOrdering$plotted
      curLayout<-curOrdering$Layout

      curRNames<-rNames(object)
      if (is(modelData, "exprSet"))
        modelData<-modelData[which(geneNames(modelData) %in% curRNames), ]
      else
        modelData<-modelData[which(rownames(modelData) %in% curRNames), ]

      virtualData<-virtualData[which(rownames(virtualData) %in% curRNames), ]
    
      controlEnv<-controller(curMVC)

      mainHMIndex<-match("mainHeatmap", unlist(curPlotted))

      curGeneName<-vData$rowName
      allGeneNames<-geneNames(modelData)

      # need to redraw the whole heatmap if any genes need to be hidden
      # this just gets the dendrogram ordering without the genes to be hidden
      # only do this if we are hiding genes (not re-showing them!)
      if (any(vData$colName=="hide") && all(vData$newValue==TRUE)) 
      {
        # note that this does not guarantee that the dendrogram ordering will 
        # be the same!!!
        curRowToHide<-match(curGeneName, allGeneNames)
        allHiddenRows<-which(virtualData$hide)
        oldHiddenRows<-allHiddenRows[!(allHiddenRows %in% curRowToHide)]
     
        if (length(oldHiddenRows) > 0)
          genesUsed<-allGeneNames[-oldHiddenRows]
        else
          genesUsed<-allGeneNames

        newRowToHide<-match(curGeneName, genesUsed)

        # now find the current value stored in the dendrogram ordering
        curVal<-curRowInd[newRowToHide]
        scurVal<-sort(curVal, decreasing=TRUE)   

        for (i in 1:length(scurVal))
        {
          # subtract one from all the numbers greater than curVal
          curRowInd[curRowInd > scurVal[i]]<-
                       curRowInd[curRowInd > scurVal[i]] -1
        }

        # then remove the value relating to the gene that was just hidden
        curRowInd<-curRowInd[-newRowToHide]
      }

      # only need to redraw a part of the heatmap
      if (any(vData$colName == "color") || any(vData$colName == "highlit"))
      {
        pDevice<-plotDevice(object)
        dev.set(pDevice)
     
        curPP<-curPlotPar[[mainHMIndex]]
        curFig<-curPP$fig
        curFig[curPP$fig<0]<-0
        curPP[["fig"]]<-curFig
        par(curPP) 
        par("fig"=curFig, new=TRUE)

        if (any(virtualData$hide))
          rowsToHide<-which(virtualData$hide)
        if (any(virtualData$hide))
          curColor<-virtualData$color[-rowsToHide]
        else
          curColor<-virtualData$color
        curHighlight<-virtualData$highlit
        curFont<-rep(1, nrow(virtualData))
        curFont[curHighlight]<-4
        if (any(virtualData$hide))
          curFont<-curFont[-rowsToHide]

        # need to get the row labels (for the genes)
        labRow<-geneNames(modelData)
        # then need to remove hidden names
        if (any(virtualData$hide))
          labRow<-labRow[-rowsToHide]
        # then need to reorder to match the dendrogram
        labRow<-labRow[curRowInd]

        iy<-1:length(labRow)
        newL<-0.5
        cexRow<-0.2 + 1/log10(length(labRow))

        if (any(vData$colName == "highlit"))
        {
          oldFont<-curFont
          if (any(virtualData$hide))
            curGenes<-allGeneNames[-rowsToHide]
          else
            curGenes<-allGeneNames
          if (any(vData$oldValue==TRUE))
          {
            oldHigh<-which(vData$oldValue==TRUE)
            oldFont[match(vData$rowName[oldHigh], curGenes)]<-4
          }
          if (any(vData$oldValue==FALSE))
          {
            oldNotHigh<-which(vData$oldValue==FALSE)
            oldFont[match(vData$rowName[oldNotHigh], curGenes)]<-1
          }
        }
        else
          oldFont<-curFont

        # need to sort color and font to match the dendrograms
        curColor<-curColor[curRowInd]
        curFont<-curFont[curRowInd]
        oldFont<-oldFont[curRowInd]

        # change affy ids to gene symbols
        if (modelData@annotation != "" || get("chipType", controlEnv) != "")
        {
          if (modelData@annotation != "")
            curSYMBENV<-paste(modelData@annotation, "SYMBOL", sep="")
          else
            curSYMBENV<-paste(get("chipType", controlEnv), "SYMBOL", sep="")
 
          curSymb<-unlist(mget(labRow, eval(as.name(curSYMBENV))))
          curSymb[which(is.na(curSymb))]<-names(curSymb[which(is.na(curSymb))])
          curText<-curSymb       
        }
        else
          curText<-labRow

        # now can remove the old values 
        mtext(text=curText, side=4, at=iy, las=2, line=newL,
            cex.axis=cexRow, col=rep("white", length(labRow)), font=oldFont)

        if (length(curText) > 50)
        {
          # then only need to show certain rows that are either colored or 
          # highlit
          # so need to reset labRow
          curText[curColor=="black" & curFont==1]<-""
        }

        # now add the new values
        mtext(text=curText, side=4, at=iy, las=2, line=newL,
            cex.axis=cexRow, col=curColor, font=curFont)

        # reset the active device (don't need to reset the MVCList)
        dev.set(curdev)
      }

      # then need to redraw the whole heatmap if hide was changed and any of 
      # the values have changed
      if (any(vData$colName == "hide") && 
              any(vData$newValue != vData$oldValue))
      {
        # set the current device
        pDevice<-plotDevice(object)
        dev.set(pDevice)

        # this will be the genes to hide
        # need to check all genes to see which ones to hide
        if (any(virtualData$hide))
        {
          rowsToHide<-which(virtualData$hide)
          newmodelData<-modelData[-rowsToHide,]
        }
        else
          newmodelData<-modelData

        # also get the highlit and color columns from the virtualData
        if (any(virtualData$hide))
          curColor<-virtualData$color[-rowsToHide]
        else
          curColor<-virtualData$color
        curHighlight<-virtualData$highlit
        curFont<-rep(1, nrow(virtualData))
        curFont[curHighlight]<-4
        if (any(virtualData$hide))
          curFont<-curFont[-rowsToHide]

        # change affy ids to gene symbols
        if (modelData@annotation != "" || get("chipType", controlEnv) != "")
        {
          if (modelData@annotation != "")
            curSYMBENV<-paste(modelData@annotation, "SYMBOL", sep="")
          else
            curSYMBENV<-paste(get("chipType", controlEnv), "SYMBOL", sep="")

          labRow<-geneNames(newmodelData) 
          curSymb<-unlist(mget(labRow, eval(as.name(curSYMBENV))))
          curSymb[which(is.na(curSymb))]<-names(curSymb[which(is.na(curSymb))])
          dataToPlot<-exprs(newmodelData)
          rownames(dataToPlot)<-curSymb
        }
        else
          dataToPlot<-exprs(newmodelData)

        # only show row and column names if there are less than 50 (otherwise
        # the names start to overlap and they can't be read)
        curlabRow<-rownames(dataToPlot)
        if (length(curlabRow) > 50)
          curlabRow[curColor=="black" & curFont==1]<-""
        curlabCol<-colnames(dataToPlot)
        if (length(curlabCol) > 50)
          curlabCol<-rep("", length(curlabCol))

        retList<-newheatmap(dataToPlot, Rowv=curRowInd,
                        margins=c(7,8), textColor=curColor, font=curFont,
                        keep.dendro=TRUE, labRow=curlabRow, labCol=curlabCol)
 
        # need to reset the ordering and parameter values in heatmapView
        ordering(object)<-retList
        plotPar(object)<-par(no.readonly=TRUE)

        curviewList<-viewList(curMVC)
        # need to reassign the current view list
        # which element in the viewList is it?
        allDevices<- unlist(lapply(curviewList, plotDevice))
        devIndex<-match(pDevice, allDevices)
        curviewList[[devIndex]]<-object

        # then need to reassign the current MVC
        viewList(curMVC)<-curviewList
        MVCList<-get("MVCList", mvcEnv)
        allNames<-getModelNames(sort=FALSE)
        mIndex<-match(dataName, allNames)
        MVCList[[mIndex]]<-curMVC
        assign("MVCList", MVCList, mvcEnv) 

        dev.set(curdev)
      }
    }
  }
)

##############
# click event and motion event methods for views
##############

########
# clickEvent for spreadView is called in response to a selected row
# or unselected row from the clist on a spreadView
########
setMethod("clickEvent", "spreadView", 
  function(object, where, ...)
  {
    extras<-list(...)
    # event is select or unselect
    event<-extras$event

    # assume that the clickEvent for spreadView has the same callback
    # function as the left button click because a row can only be selected
    # or unselected with the left button

    # may want to check if the view clicked on is from the activeMVC
    # 5/31/06 respond to any events (even if the view does not belong to the
    # active MVC)
    activeMVC<-get("activeMVC", mvcEnv)
#    if (dataName(object) == activeMVC)
#    {
      curMVC<-getMVC(dataName(object))
      controlEnv<-controller(curMVC)
      curModel<-model(curMVC)
      modelData<-modelData(curModel)
      numCListRows<-gtkCListGetRows(clist(object))
      firstRowText<-gtkCListGetText(clist(object), 0:(numCListRows-1), 0)

      if (is(curModel, "dfModel"))
      {
#        selValue<-rownames(modelData)[where]
        selValue<-firstRowText[where]
      }
      if (is(curModel, "graphModel"))
      {
#        selValue<-nodes(modelData)[where]
        selValue<-firstRowText[where]
      }
      if (is(curModel, "gseModel"))
      {
#        selValue<-rownames(incidMat(modelData))[where]
        selValue<-firstRowText[where]
      }

      # see if there is a function to call
      leftButtonClick<-get("leftButtonClick", controlEnv)

      # see if any of the functions are active
      activeLBCFun<-unlist(lapply(leftButtonClick, function(y)
        {y$assigned==TRUE}))

      curIndex<-which(activeLBCFun)
      if (length(curIndex)==1)
      { 
        curEventFun<-leftButtonClick[[curIndex]][["eventFun"]]

        actualFun<-callFun(curEventFun)
        # problem is that we can't list it as select or unselect event
        # but that's ok because highlight and hide will be toggled anyway
        do.call(actualFun, list(selValue))
      }
#    }
    # return TRUE if there are no errors because gtkAddCallback is expecting
    # a boolean
    return(TRUE)
  }
)

#########
# create a button press event for scatterplot views (used to be
# addButtonPress function)
#########
setMethod("clickEvent", "sPlotView",
  function(object, where, ...)
  {
    # may want to check if the view clicked on is from the activeMVC
    # 5/31/06 respond to any events (even if the view does not belong to the
    # active MVC)
    activeMVC<-get("activeMVC", mvcEnv)
#    if (dataName(object) == activeMVC)
#    {
      curMVC<-getMVC(dataName(object))
      controlEnv<-controller(curMVC)

      # see if there is a function to call
      rightButtonClick<-get("rightButtonClick", controlEnv)
      leftButtonClick<-get("leftButtonClick", controlEnv)
      middleButtonClick<-get("middleButtonClick", controlEnv)

      # see if any of the functions are active
      activeRBCFun<-unlist(lapply(rightButtonClick, function(y)
        {y$assigned==TRUE}))
      activeLBCFun<-unlist(lapply(leftButtonClick, function(y)
        {y$assigned==TRUE}))
      activeMBCFun<-unlist(lapply(middleButtonClick, function(y)
        {y$assigned==TRUE}))

      # where will be the mouse click event information from Gtk
      # check that the left button was pressed
      if (gdkEventButtonGetButton(where)==1)
      {
        if (any(activeLBCFun))
        {
          usri <- pix2inches(where[["X"]], where[["Y"]])
          usrc <- inches2usr(usri$x, usri$y)

          curPoint<-identifyView(object=object, location=usrc)

          if (!is.null(curPoint))
          {
            # if a point was clicked, then we want to create a 
            # gUpdateDataMessage this is done in the active eventFun function

            curIndex<-which(activeLBCFun)
            if (length(curIndex)==1)
            { 
              curEventFun<-leftButtonClick[[curIndex]][["eventFun"]]

              # now call the function the user has set with the current
              # point as the parameter
              actualFun<-callFun(curEventFun)
              do.call(actualFun, list(curPoint$closestPoint))
            }
          }
        }
      }

      if (gdkEventButtonGetButton(where)==2)
      {
        if (any(activeMBCFun))
        {
          usri <- pix2inches(where[["X"]], where[["Y"]])
          usrc <- inches2usr(usri$x, usri$y)

          curPoint<-identifyView(object=object, location=usrc)

          if (!is.null(curPoint))
          {
            curIndex<-which(activeMBCFun)
            if (length(curIndex)==1)
            { 
              curEventFun<-middleButtonClick[[curIndex]][["eventFun"]]

              # now call the function the user has set with the current
              # point as the parameter 
              actualFun<-callFun(curEventFun)
              do.call(actualFun, list(curPoint$closestPoint))
            }
          }
        }
      }

      if (gdkEventButtonGetButton(where)==3)
      {
        if (any(activeRBCFun))
        {
          usri <- pix2inches(where[["X"]], where[["Y"]])
          usrc <- inches2usr(usri$x, usri$y)

          curPoint<-identifyView(object=object, location=usrc)

          if (!is.null(curPoint))
          {
            curIndex<-which(activeRBCFun)
            if (length(curIndex)==1)
            {
              curEventFun<-rightButtonClick[[curIndex]][["eventFun"]]
  
              # now call the function the user has set with the current
              # point as the parameter
              actualFun<-callFun(curEventFun)
              do.call(actualFun, list(curPoint$closestPoint))
            }
          }
        }
      }
#    }
    # return TRUE if there are no errors because gtkAddCallback is expecting
    # a boolean
    return(TRUE)
  }
)

setMethod("clickEvent", "qqPlotView",
  function(object, where, ...)
  {
    # respond to any events (even if the view does not belong to the
    # active MVC)
    curMVC<-getMVC(dataName(object))
    controlEnv<-controller(curMVC)

    # see if there is a function to call
    rightButtonClick<-get("rightButtonClick", controlEnv)
    leftButtonClick<-get("leftButtonClick", controlEnv)
    middleButtonClick<-get("middleButtonClick", controlEnv)

    # see if any of the functions are active
    activeRBCFun<-unlist(lapply(rightButtonClick, function(y)
      {y$assigned==TRUE}))
    activeLBCFun<-unlist(lapply(leftButtonClick, function(y)
      {y$assigned==TRUE}))
    activeMBCFun<-unlist(lapply(middleButtonClick, function(y)
      {y$assigned==TRUE}))

    # where will be the mouse click event information from Gtk
    # check that the left button was pressed
    if (gdkEventButtonGetButton(where)==1)
    {
      if (any(activeLBCFun))
      {
        usri <- pix2inches(where[["X"]], where[["Y"]])
        usrc <- inches2usr(usri$x, usri$y)

        curPoint<-identifyView(object=object, location=usrc)

        if (!is.null(curPoint))
        {
          # if a point was clicked, then we want to create a 
          # gUpdateDataMessage this is done in the active eventFun function

          curIndex<-which(activeLBCFun)
          if (length(curIndex)==1)
          { 
            curEventFun<-leftButtonClick[[curIndex]][["eventFun"]]

            # now call the function the user has set with the current
            # point as the parameter
            actualFun<-callFun(curEventFun)
            do.call(actualFun, list(curPoint$closestPoint))
          }
        }
      }
    }

    if (gdkEventButtonGetButton(where)==2)
    {
      if (any(activeMBCFun))
      {
        usri <- pix2inches(where[["X"]], where[["Y"]])
        usrc <- inches2usr(usri$x, usri$y)

        curPoint<-identifyView(object=object, location=usrc)

        if (!is.null(curPoint))
        {
          curIndex<-which(activeMBCFun)
          if (length(curIndex)==1)
          { 
            curEventFun<-middleButtonClick[[curIndex]][["eventFun"]]

            # now call the function the user has set with the current
            # point as the parameter 
            actualFun<-callFun(curEventFun)
            do.call(actualFun, list(curPoint$closestPoint))
          }
        }
      }
    }

    if (gdkEventButtonGetButton(where)==3)
    {
      if (any(activeRBCFun))
      {
        usri <- pix2inches(where[["X"]], where[["Y"]])
        usrc <- inches2usr(usri$x, usri$y)
        curPoint<-identifyView(object=object, location=usrc)

        if (!is.null(curPoint))
        {
          curIndex<-which(activeRBCFun)
          if (length(curIndex)==1)
          {
            curEventFun<-rightButtonClick[[curIndex]][["eventFun"]]
  
            # now call the function the user has set with the current
            # point as the parameter
            actualFun<-callFun(curEventFun)
            do.call(actualFun, list(curPoint$closestPoint))
          }
        }
      }
    }

    # return TRUE if there are no errors because gtkAddCallback is expecting
    # a boolean
    return(TRUE)
  }
)

########
# a method for a motion notify event on a scatterplot
########
setMethod("motionEvent", "sPlotView",
  function(object, event, ...)
  {
    # may want to check if the view clicked on is from the activeMVC
    # 5/31/06 respond to any events (even if the view does not belong to the
    # active MVC)
    activeMVC<-get("activeMVC", mvcEnv)
#    if (dataName(object) == activeMVC)
#    {
      curMVC<-getMVC(dataName(object))
      controlEnv<-controller(curMVC)

      # see if there is a function to call
      mouseOver<-get("mouseOver", controlEnv)
      # see if any of the functions are active
      activeFun<-unlist(lapply(mouseOver, function(y){y$assigned==TRUE}))

      # then there is an active function
      if (any(activeFun))
      {
        curWin<-win(object)
        # potential problem - these depend on how the 
        # scatterplot window is set up
        # SHOULD AT LEAST CHECK THEIR CLASSES!!
        hrul<-gtkContainerGetChildren(
             gtkContainerGetChildren(curWin)[[1]])[[1]]
        vrul<-gtkContainerGetChildren(gtkContainerGetChildren(
             gtkContainerGetChildren(curWin)[[1]])[[2]])[[1]]

        hrul$SignalEmit("motion-notify-event", event)
        vrul$SignalEmit("motion-notify-event", event)
      
        curx<-gdkEventMotionGetX(event)
        cury<-gdkEventMotionGetY(event)

        # convert to user coordinates
        xyInches<-pix2inches(curx, cury)
        xyUsr<-inches2usr(xyInches$x, xyInches$y)
 
        # also need to ensure that the plot has been added to viewList
        curDev<-dev.cur()
        curViewList<-viewList(curMVC)

        devicesInView<-unlist(lapply(curViewList, function(x) {is(x,
                             "plotView")}))
        # only look at views that have devices
        viewsWithDevices<-curViewList[devicesInView]
        plotDev<-unlist(lapply(viewsWithDevices, function(x) {plotDevice(x)}))

        if (curDev %in% plotDev)
        {
          curPoint<-identifyView(object=object, location=xyUsr)

          curIndex<-which(activeFun)
          if (length(curIndex)==1)
          {
            curEventFun<-mouseOver[[curIndex]][["eventFun"]]

            # now call the function the user has set with the current
            # point as the parameter - assume the function does not return
            # anything to the motion notify event
            actualFun<-callFun(curEventFun)
            do.call(actualFun, list(curPoint$closestPoint))
          }
        }
      }
#    }
    # return TRUE if there are no errors because gtkAddCallback is expecting
    # a boolean
    return(TRUE)
  }
)

setMethod("motionEvent", "qqPlotView",
  function(object, event, ...)
  {
    # respond to any events (even if the view does not belong to the
    # active MVC)
    curMVC<-getMVC(dataName(object))
    controlEnv<-controller(curMVC)

    # see if there is a function to call
    mouseOver<-get("mouseOver", controlEnv)
    # see if any of the functions are active
    activeFun<-unlist(lapply(mouseOver, function(y){y$assigned==TRUE}))

    # then there is an active function
    if (any(activeFun))
    {
      curWin<-win(object)
      # potential problem - these depend on how the 
      # scatterplot window is set up
      # SHOULD AT LEAST CHECK THEIR CLASSES!!
      hrul<-gtkContainerGetChildren(
             gtkContainerGetChildren(curWin)[[1]])[[1]]
      vrul<-gtkContainerGetChildren(gtkContainerGetChildren(
             gtkContainerGetChildren(curWin)[[1]])[[2]])[[1]]

      hrul$SignalEmit("motion-notify-event", event)
      vrul$SignalEmit("motion-notify-event", event)

      curx<-gdkEventMotionGetX(event)
      cury<-gdkEventMotionGetY(event)

      # convert to user coordinates
      xyInches<-pix2inches(curx, cury)
      xyUsr<-inches2usr(xyInches$x, xyInches$y)
 
      # also need to ensure that the plot has been added to viewList
      curDev<-dev.cur()
      curViewList<-viewList(curMVC)

      devicesInView<-unlist(lapply(curViewList, function(x) {is(x,
                             "plotView")}))
      # only look at views that have devices
      viewsWithDevices<-curViewList[devicesInView]
      plotDev<-unlist(lapply(viewsWithDevices, function(x) {plotDevice(x)}))

      if (curDev %in% plotDev)
      {
        curPoint<-identifyView(object=object, location=xyUsr)

        curIndex<-which(activeFun)
        if (length(curIndex)==1)
        {
          curEventFun<-mouseOver[[curIndex]][["eventFun"]]

          # now call the function the user has set with the current
          # point as the parameter - assume the function does not return
          # anything to the motion notify event
          actualFun<-callFun(curEventFun)
          do.call(actualFun, list(curPoint$closestPoint))
        }
      }
    }
    # return TRUE if there are no errors because gtkAddCallback is expecting
    # a boolean
    return(TRUE)
  }
)

##########
# 4/22/06
##########
setMethod("clickEvent", "heatmapView", 
  function(object, where, ...)
  {
    # may want to check if the view clicked on is from the activeMVC
    # 5/31/06 respond to any events (even if the view does not belong to the
    # active MVC)
    activeMVC<-get("activeMVC", mvcEnv)
#    if (dataName(object) == activeMVC)
#    {
      curMVC<-getMVC(dataName(object))
      controlEnv<-controller(curMVC)

      ###########
      # 4/26/06 
      # the curView object used for the click event is set when the view is 
      # initialized so it is never updated
      # thus, we need to reset the object to its current value that
      # is stored in the MVCList variable
      ###########
      curDevice<-plotDevice(object)
      allDevices<-unlist(lapply(viewList(curMVC), "plotDevice"))
      plotIndex<-match(curDevice, allDevices)
      object<-viewList(curMVC)[[plotIndex]]
      
      # see if there is a function to call
      rightButtonClick<-get("rightButtonClick", controlEnv)
      leftButtonClick<-get("leftButtonClick", controlEnv)
      middleButtonClick<-get("middleButtonClick", controlEnv)

      # see if any of the functions are active
      activeRBCFun<-unlist(lapply(rightButtonClick, function(y)
        {y$assigned==TRUE}))
      activeLBCFun<-unlist(lapply(leftButtonClick, function(y)
        {y$assigned==TRUE}))
      activeMBCFun<-unlist(lapply(middleButtonClick, function(y)
        {y$assigned==TRUE}))

      # where will be the mouse click event information from Gtk
      # check that the left button was pressed
      if (gdkEventButtonGetButton(where)==1)
      {
        if (any(activeLBCFun))
        {
          curWin<-win(object)

          # convert to user coordinates
          xyIn<-pix2inches(where[["X"]], where[["Y"]])      
          curGene<-identifyView(object, xyIn)
        
          if (!is.null(curGene))
          {
            curIndex<-which(activeLBCFun)
            if (length(curIndex)==1)
            { 
              curEventFun<-leftButtonClick[[curIndex]][["eventFun"]]

              # now call the function the user has set with the current
              # gene as the parameter
              actualFun<-callFun(curEventFun)
              do.call(actualFun, list(curGene))
            }
          }
        }
      }
      if (gdkEventButtonGetButton(where)==2)
      {
        if (any(activeMBCFun))
        {
          curWin<-win(object)

          # convert to user coordinates
          xyIn<-pix2inches(where[["X"]], where[["Y"]]) 
          curGene<-identifyView(object, xyIn)
          if (!is.null(curGene))
          {
            curIndex<-which(activeMBCFun)
            if (length(curIndex)==1)
            { 
              curEventFun<-middleButtonClick[[curIndex]][["eventFun"]]

              # now call the function the user has set with the current
              # gene as the parameter
              actualFun<-callFun(curEventFun)
              do.call(actualFun, list(curGene))
            }
          }
        }
      }
      if (gdkEventButtonGetButton(where)==3)
      {
        if (any(activeRBCFun))
        {
          curWin<-win(object)

          # convert to user coordinates
          xyIn<-pix2inches(where[["X"]], where[["Y"]]) 
          curGene<-identifyView(object, xyIn)
          if (!is.null(curGene))
          {
            curIndex<-which(activeRBCFun)
            if (length(curIndex)==1)
            { 
              curEventFun<-rightButtonClick[[curIndex]][["eventFun"]]

              # now call the function the user has set with the current
              # gene as the parameter
              actualFun<-callFun(curEventFun)
              do.call(actualFun, list(curGene))
            }
          }
        }
      }
#    }
    # return TRUE if there are no errors because gtkAddCallback is expecting
    # a boolean
    return(TRUE)
  }
)

##########
# 7/27/05
##########
setMethod("clickEvent", "graphView",
  function(object, where, ...)
  {
    # may want to check if the view clicked on is from the activeMVC
    # 5/31/06 respond to any events (even if the view does not belong to the
    # active MVC)
    activeMVC<-get("activeMVC", mvcEnv)
#    if (dataName(object) == activeMVC)
#    {
      curMVC<-getMVC(dataName(object))
      controlEnv<-controller(curMVC)
      curViewLayout<-grLayout(object)

      # see if there is a function to call
      rightButtonClick<-get("rightButtonClick", controlEnv)
      leftButtonClick<-get("leftButtonClick", controlEnv)
      middleButtonClick<-get("middleButtonClick", controlEnv)

      # see if any of the functions are active
      activeRBCFun<-unlist(lapply(rightButtonClick, function(y)
        {y$assigned==TRUE}))
      activeLBCFun<-unlist(lapply(leftButtonClick, function(y)
        {y$assigned==TRUE}))
      activeMBCFun<-unlist(lapply(middleButtonClick, function(y)
        {y$assigned==TRUE}))

      # where will be the mouse click event information from Gtk
      # check that the left button was pressed
      if (gdkEventButtonGetButton(where)==1)
      {
        if (any(activeLBCFun))
        {
          curWin<-win(object)

          # convert to user coordinates
          xyUsr<-pix2usr(where[["X"]], where[["Y"]])      
          curNode<-identifyView(object, xyUsr)

          if (!is.null(curNode))
          {
            curIndex<-which(activeLBCFun)
            if (length(curIndex)==1)
            { 
              curEventFun<-leftButtonClick[[curIndex]][["eventFun"]]

              # now call the function the user has set with the current
              # point as the parameter
              actualFun<-callFun(curEventFun)
              do.call(actualFun, list(name(curNode)))
            }
          }
        }
      }
      if (gdkEventButtonGetButton(where)==2)
      {
        if (any(activeMBCFun))
        {
          curWin<-win(object)

          # convert to user coordinates
          xyUsr<-pix2usr(where[["X"]], where[["Y"]])      
          curNode<-identifyView(object, xyUsr)

          if (!is.null(curNode))
          {
            curIndex<-which(activeMBCFun)
            if (length(curIndex)==1)
            { 
              curEventFun<-middleButtonClick[[curIndex]][["eventFun"]]

              # now call the function the user has set with the current
              # point as the parameter
              actualFun<-callFun(curEventFun)
              do.call(actualFun, list(name(curNode)))
            }
          }
        }
      }
      if (gdkEventButtonGetButton(where)==3)
      {
        if (any(activeRBCFun))
        {
          curWin<-win(object)

          # convert to user coordinates
          xyUsr<-pix2usr(where[["X"]], where[["Y"]])      
          curNode<-identifyView(object, xyUsr)

          if (!is.null(curNode))
          {
            curIndex<-which(activeRBCFun)
            if (length(curIndex)==1)
            { 
              curEventFun<-rightButtonClick[[curIndex]][["eventFun"]]

              # now call the function the user has set with the current
              # point as the parameter
              actualFun<-callFun(curEventFun)
              do.call(actualFun, list(name(curNode)))
            }
          }
        }
      }
#    }
    # return TRUE if there are no errors because gtkAddCallback is expecting
    # a boolean
    return(TRUE)
  }
)

#############
# added 6/28/05 so that the graphView has a motionEvent
#############
setMethod("motionEvent", "graphView",
  function(object, event, ...)
  {
    # may want to check if the view clicked on is from the activeMVC
    # 5/31/06 respond to any events (even if the view does not belong to the
    # active MVC)
    activeMVC<-get("activeMVC", mvcEnv)
#    if (dataName(object) == activeMVC)
#    {
      curMVC<-getMVC(dataName(object))
      controlEnv<-controller(curMVC)
      curLayout<-grLayout(object)

      # see if there is a function to call
      mouseOver<-get("mouseOver", controlEnv)
      # see if any of the functions are active
      activeFun<-unlist(lapply(mouseOver, function(y){y$assigned==TRUE}))

      # then there is an active function
      if (any(activeFun))
      {
        curWin<-win(object)
        # potential problem - these depend on how the 
        # plot window is set up
        # SHOULD AT LEAST CHECK THEIR CLASSES!!
        hrul<-gtkContainerGetChildren(
             gtkContainerGetChildren(curWin)[[1]])[[1]]
        vrul<-gtkContainerGetChildren(gtkContainerGetChildren(
             gtkContainerGetChildren(curWin)[[1]])[[2]])[[1]]

        hrul$SignalEmit("motion-notify-event", event)
        vrul$SignalEmit("motion-notify-event", event)
      
        curx<-gdkEventMotionGetX(event)
        cury<-gdkEventMotionGetY(event)

        # convert to user coordinates
        xyUsr<-pix2usr(curx, cury)

        # also need to ensure that the plot has been added to viewList
        curDev<-dev.cur()
        curViewList<-viewList(curMVC)

        devicesInView<-unlist(lapply(curViewList, function(x) {is(x,
                             "plotView")}))
        # only look at views that have devices
        viewsWithDevices<-curViewList[devicesInView]
        plotDev<-unlist(lapply(viewsWithDevices, function(x) {plotDevice(x)}))

        if (curDev %in% plotDev)
        {
          curNode<-identifyView(object, xyUsr)

          curIndex<-which(activeFun)
          if (length(curIndex)==1)
          { 
            curEventFun<-mouseOver[[curIndex]][["eventFun"]]

            # now call the function the user has set with the current
            # node as the parameter - assume the function does not return
            # anything to the motion notify event
            actualFun<-callFun(curEventFun)
            if (!is.null(curNode))
              do.call(actualFun, list(name(curNode)))
            else
              do.call(actualFun, list(curNode))
          }
        }
      }
#    }
    # return TRUE if there are no errors because gtkAddCallback is expecting
    # a boolean
    return(TRUE)
  }
)

############
# identifyView methods for views
# will identify an object on the view
############
setMethod("identifyView", "graphView",
  function(object, location, ...)
  {
    curName<-dataName(object)
    curModel<-model(getMVC(curName))
    # want to determine if a node was under the cursor by looking at the
    # Ragraph object stored with the view
    vplot<-grLayout(object)

    # check if vplot is valid
    if (is.null(vplot))
      stop("The layout variable must be from a graph plot")

    # check the class of the layout variable (vplot)
    if (class(vplot)!= "Ragraph")
      stop ("The layout variable must be from a graph plot")

    # need to get the location, which is the center
    nodeInfo<-AgNode(vplot)

    nodeCenters<-lapply(nodeInfo, function(x){getNodeCenter(x)})
    xCenter<-unlist(lapply(nodeCenters, function(x){getX(x)}))
    yCenter<-unlist(lapply(nodeCenters, function(x){getY(x)}))

    nodeNames<-unlist(lapply(nodeInfo, function(x){name(x)}))

    # also need widths and heights
    nodeHeight<-(unlist(lapply(nodeInfo, function(x){getNodeHeight(x)})))/2
    nodeLW<-unlist(lapply(nodeInfo, function(x){getNodeLW(x)}))
    nodeRW<-unlist(lapply(nodeInfo, function(x){getNodeRW(x)}))

    totdiff<-(location$x-xCenter)^2+(location$y-yCenter)^2
    booClosest<-min(totdiff)==totdiff

    # get the values for the closest node
    closestAgNode<-nodeInfo[booClosest][[1]]
    closestNode<-nodeNames[booClosest]
    clos.nLoc<-list(x=xCenter[booClosest], y=yCenter[booClosest])
    nHeight<-nodeHeight[booClosest]
    nLW<-nodeLW[booClosest]
    nRW<-nodeRW[booClosest]

    x.within<-FALSE
    y.within<-FALSE

    if ((location$x<=clos.nLoc$x) && (location$x>=clos.nLoc$x-nLW))
      x.within<-TRUE
    if ((location$x>clos.nLoc$x) && (location$x<=clos.nLoc$x+nRW))
      x.within<-TRUE  

    if ((location$y<=clos.nLoc$y) && (location$y>=clos.nLoc$y-nHeight))
      y.within<-TRUE
    if ((location$y>clos.nLoc$y) && (location$y<=clos.nLoc$y+nHeight))
      y.within<-TRUE 
  
    if (x.within && y.within)
    {
      # return an object of class AgNode
      return(closestAgNode)
    }
    else
      return(NULL)
  }
)

setMethod("identifyView", "sPlotView", 
  function(object, location, ...)
  {
    # get the cxy of the active plot
    cxy<-par("cxy")
    pointWidth<-cxy[1]
    pointHeight<-cxy[2]  

    plotDf<-dataName(object)

    # these will be the row and column names
    plotRows<-dfRows(object)
    plotX<-xvar(object)
    plotY<-yvar(object)

    # need to get the data from the model
    mvcList<-get("MVCList", mvcEnv)
  
    curData<-getData(modelName=plotDf)
    rowIndices<-match(plotRows, rownames(curData))
    colxIndex<-match(plotX, colnames(curData))
    colyIndex<-match(plotY, colnames(curData))

    x<-curData[rowIndices, colxIndex]
    y<-curData[rowIndices, colyIndex]

    # check if x or y is a factor
    if (is(x, "factor"))
      x<-as.numeric(x)
    if (is(y, "factor"))
      y<-as.numeric(y)    

    rowNames<-rownames(curData)

    # now need to identify the closest point
    totdiff<-(location$x-x)^2+(location$y-y)^2
    booClosest<-min(totdiff)==totdiff

    closestPoint<-rowNames[booClosest]
    closestXY<-list(x=x[booClosest], y=y[booClosest])

    # need to ensure that the closestPoint was actually clicked
    # use cxy
    x.within<-FALSE
    y.within<-FALSE
  
    if ((location$x<=closestXY$x) && (location$x>=closestXY$x-(pointWidth/2)))
      x.within<-TRUE
    if ((location$x>closestXY$x) && (location$x<=closestXY$x+(pointWidth/2)))
      x.within<-TRUE  

    if ((location$y<=closestXY$y) && (location$y>=closestXY$y-(pointHeight/2)))
      y.within<-TRUE
    if ((location$y>closestXY$y) && (location$y<=closestXY$y+(pointHeight/2)))
      y.within<-TRUE 

    if (x.within && y.within)
      return(list(closestXY=closestXY, closestPoint=closestPoint))
    else
      return(NULL)
  }
)

setMethod("identifyView", "qqPlotView", 
  function(object, location, ...)
  {
    # get the cxy of the active plot
    cxy<-par("cxy")
    pointWidth<-cxy[1]
    pointHeight<-cxy[2]  

    plotD<-dataName(object)

    allxval<-xval(object)
    allyval<-yval(object)

    # need to get the data from the model  
    curData<-getData(modelName=plotD)

    # check if x or y is a factor
    if (is(allxval, "factor"))
      allxval<-as.numeric(allxval)
    if (is(allyval, "factor"))
      allyval<-as.numeric(allyval)    

    gsNames<-rownames(incidMat(curData))

    # now need to identify the closest point
    totdiff<-(location$x-allxval)^2+(location$y-allyval)^2
    booClosest<-min(totdiff)==totdiff

    closestPoint<-gsNames[booClosest]
    closestXY<-list(x=allxval[booClosest], y=allyval[booClosest])

    # need to ensure that the closestPoint was actually clicked
    # use cxy
    x.within<-FALSE
    y.within<-FALSE
  
    if ((location$x<=closestXY$x) && (location$x>=closestXY$x-(pointWidth/2)))
      x.within<-TRUE
    if ((location$x>closestXY$x) && (location$x<=closestXY$x+(pointWidth/2)))
      x.within<-TRUE  

    if ((location$y<=closestXY$y) && (location$y>=closestXY$y-(pointHeight/2)))
      y.within<-TRUE
    if ((location$y>closestXY$y) && (location$y<=closestXY$y+(pointHeight/2)))
      y.within<-TRUE 

    if (x.within && y.within)
      return(list(closestXY=closestXY, closestPoint=closestPoint))
    else
      return(NULL)
  }
)

setMethod("identifyView", "heatmapView",
  function(object, location, ...)
  {
    # location will be in inches
    # need to get the information from MVCList because it is current
    MVCName<-dataName(object)
    curMVC<-getMVC(MVCName)
    curVirData<-virtualData(model(curMVC))

    curRNames<-rNames(object)
    curVirData<-curVirData[which(rownames(curVirData) %in% curRNames), ]

    curHidden<-curVirData$hide

    curOrdering<-ordering(object)
    curLayout<-curOrdering$Layout
    curPlotted<-curOrdering$plotted
    curLayoutMat<-curLayout$layoutMat
    curPlotPar<-curOrdering$plotPar

    # din is the device dimensions in inches (width, height)
    devWidth<-par("din")[1]
    devHeight<-par("din")[2]

    # first divide the plot into different sections and figure out where
    # the event occurred
    curLayoutWidth<-curLayout$width
    curLayoutHeight<-curLayout$height
    totWidthDiv<-sum(curLayoutWidth)
    totHeightDiv<-sum(curLayoutHeight)
        
    locx<-location$x
    locy<-location$y

    inBox<-c(0, 0)
    for (i in 1:length(curLayoutWidth))
    {
      for (j in length(curLayoutHeight):1)
      {
        # check the first section on the bottom left
        if (i==1)
          left<-0
        else
        {
          widthSoFar<-sum(curLayoutWidth[1:(i-1)])
          left<-devWidth*(widthSoFar/totWidthDiv)
        }
        if (j==length(curLayoutHeight))
          bot<-0
        else
        {
          heightSoFar<-sum(curLayoutHeight[length(curLayoutHeight):(j+1)])
          bot<-devHeight*(heightSoFar/totHeightDiv)
        }
        
        widthSoFar<-sum(curLayoutWidth[1:i])
        right<-devWidth*(widthSoFar/totWidthDiv)
     
        heightSoFar<-sum(curLayoutHeight[length(curLayoutHeight):j])
        top<-devHeight*(heightSoFar/totHeightDiv)

        xwithin<-FALSE
        ywithin<-FALSE
        if (locx >= left & locx <= right)
          xwithin<-TRUE
        if (locy >= bot & locy <= top)
          ywithin<-TRUE

        if (xwithin & ywithin)
        {
          currentBox<-c(bot, left, top, right)
          # j indexes height so it refers to rows and i
          # indexes width so it refers to columns
          # so it will be (row, column)
          inBox<-c(j, i)
          boxWidth<-right-left
          boxHeight<-top-bot
        }
      }
    }
 
    # now once we know which section of the layout the user clicked in,
    # we need to figure out if they user clicked on an object in that section

    curLayoutNumber<-curLayoutMat[inBox[1], inBox[2]]

    # now figure out if anything is plotted there
    # if layout number is 0, then nothing is plotted in that section
    if (curLayoutNumber != 0)
    {
      curPlotValue<-curPlotted[[curLayoutNumber]]
      # now need to look at the margins
      curMarginValuesinInches<-curPlotPar[[curLayoutNumber]][["mai"]]

      needToReset<-FALSE
      if (any(curPlotPar[[curLayoutNumber]][["fin"]] > par("din")))
      {
        # we have a problem - fin needs to be reset
        minVal<-min(boxWidth, boxHeight)
        curFin<-c(minVal, minVal)
        curPlotPar[[curLayoutNumber]][["fin"]]<-curFin
        needToReset<-TRUE
      }
      else
        curFin<-curPlotPar[[curLayoutNumber]][["fin"]]
      if (any(curPlotPar[[curLayoutNumber]][["pin"]] > par("din")))
      {
        # we have a problem - pin needs to be reset
        pinW<-curFin[1] - curMarginValuesinInches[2] - 
                          curMarginValuesinInches[4] 
        pinH<-curFin[2] - curMarginValuesinInches[1] -
                          curMarginValuesinInches[3]
        curPin<-c(pinW, pinH)
        curPlotPar[[curLayoutNumber]][["pin"]]<-curPin
        needToReset<-TRUE
      }
      else
        curPin<-curPlotPar[[curLayoutNumber]][["pin"]]

      if (needToReset)
      {
        # then reset plt
        curx1<-curMarginValuesinInches[2]/curFin[1]
        curx2<-1-(curMarginValuesinInches[4]/curFin[1])
        cury1<-curMarginValuesinInches[1]/curFin[2]
        cury2<-1-(curMarginValuesinInches[3]/curFin[2])
        curPlt<-c(curx1, curx2, cury1, cury2)
        curPlotPar[[curLayoutNumber]][["plt"]]<-curPlt

        curOrdering$plotPar[[curLayoutNumber]]<-curPlotPar[[curLayoutNumber]]
        ordering(object)<-curOrdering
        
        curviewList<-viewList(curMVC)
        # need to reassign the current view list
        # which element in the viewList is it?
        allDevices<- unlist(lapply(curviewList, plotDevice))
        devIndex<-match(dev.cur(), allDevices)
        curviewList[[devIndex]]<-object

        # then need to reassign the current MVC
        viewList(curMVC)<-curviewList
        MVCList<-get("MVCList", mvcEnv)
        allNames<-getModelNames(sort=FALSE)
        mIndex<-match(MVCName, allNames)
        MVCList[[mIndex]]<-curMVC
        assign("MVCList", MVCList, mvcEnv) 
      }

      # now see if the user clicked on a margin or on an object
      xwithin<-FALSE
      ywithin<-FALSE
      newBoxValues<-c(currentBox[1]+curMarginValuesinInches[1],
                      currentBox[2]+curMarginValuesinInches[2],
                      currentBox[3]-curMarginValuesinInches[3],
                      currentBox[4]-curMarginValuesinInches[4])

      # between left and right
      if (locx >= newBoxValues[2] & locx <= newBoxValues[4])
        xwithin<-TRUE
      # between bottom and top
      if (locy >= newBoxValues[1] & locy <= newBoxValues[3])
        ywithin<-TRUE               

      if (xwithin & ywithin)
      {
        # now determine which 'object' the user wants to identify
        # for now assume they only want to identify genes (not samples!)
        if (curPlotValue == "mainHeatmap" || curPlotValue == "rowHeatmap")
        {
          # return the gene name
          # how many genes are there?
          gN<-curRNames
#          gN<-geneNames(getData(dataName(object)))

          curplotgN<-gN[!curHidden]
          curnRows<-length(curplotgN)

          # (top-bottom)/(number of rows)
          heightPerRow<-(newBoxValues[3]-newBoxValues[1])/curnRows
          identifyRow<-as.integer((locy-newBoxValues[1])/heightPerRow)

          # need to round up by one (because the first box will correspond
          # with a value of zero due to the subtraction above)
          # return the gene name
          return(curplotgN[curOrdering$rowInd[identifyRow+1]])
        }
        if (curPlotValue == "colHeatmap")
        {
          # return the sample name (not sure want to implement this!)
          sN<-sampleNames(getData(dataName(object)))
          curnCols<-length(sN)
          # (right-left)/(number of columns)
          widthPerCol<-(newBoxValues[4]-newBoxValues[2])/curnCols
          identifyCol<-as.integer((locx-newBoxValues[2])/widthPerCol)
          # need to round up by one (because the first box will correspond
          # with a value of zero due to the subtraction above)
          # for now don't return the sample name
          print(sN[curOrdering$colInd[identifyCol+1]])
          return(NULL)
        }
        if (curPlotValue == "colDendrogram")
        {
          # would return sample names
          curUsr<-curPlotPar[[curLayoutNumber]][["usr"]]
          # pin is width followed by height
          curPin<-c(newBoxValues[4]-newBoxValues[2], 
                      newBoxValues[3]-newBoxValues[1])
          curMai<-curPlotPar[[curLayoutNumber]][["mai"]]
          xu <- curUsr[1] + (locx-newBoxValues[2])*
                            (curUsr[2]-curUsr[1])/curPin[1]
          yu <- curUsr[3] + (locy-newBoxValues[1])*
                            (curUsr[4]-curUsr[3])/curPin[2]

          #  in the margins if
          if (yu < 0)
            yu<-0
   
          # now call the identify function
          dendNames<-myDendroIdentify(locat=list(x=xu, y=yu), 
                                      dendro=curOrdering$Colv)
          print(dendNames)
          return(NULL)
        }
        if (curPlotValue == "rowDendrogram")
        {
          # return gene names
          # locx and locy are in inches - need to convert to user
          # coordinates
          curUsr<-curPlotPar[[curLayoutNumber]][["usr"]]
          # pin is width followed by height
          curPin<-c(newBoxValues[4]-newBoxValues[2], 
                    newBoxValues[3]-newBoxValues[1])
          curMai<-curPlotPar[[curLayoutNumber]][["mai"]]

          # the user coordinates from x are zeroed from the RHS (rather
          # than the LHS - so we need to subtract the x from the width)
          xu <- curUsr[2] + (curPin[1]-(locx-newBoxValues[2]))*
                            (curUsr[1]-curUsr[2])/curPin[1]
          yu <- curUsr[3] + (locy-newBoxValues[1])*
                            (curUsr[4]-curUsr[3])/curPin[2]
          # in the margins
          if (xu < 0)
            xu<-0
          
          # then need to switch and x and y because it is rotated
          # now call the identify function
          dendNames<-myDendroIdentify(locat=list(x=yu, y=xu), 
                                      dendro=curOrdering$Rowv)
          # return the affy ids (not the gene symbols!)
          return(names(dendNames))
        }
      }
    } 
    return(NULL)
  }
)


############
# initialize methods for views
############

setMethod("initialize", "qqPlotView",
  function(.Object, ...)
  {
    dataName<-list(...)$dataName
    
    retList<-initSPlotView()

    .Object@dataName<-dataName
    .Object@win<-retList$win
    .Object@plotDevice<-retList$plotDevice
    .Object@plotPar<-retList$plotPar
    .Object@drArea<-retList$drArea

    # create the actual plot
    # xval and yval slots are filled in this function
    .Object<-makeQQplot(.Object)

    # finally add events to the plot
    .Object<-addEventsforViews(.Object)
    addEventsforSPlots(.Object)

    .Object
  }
)

# dataName, dfRows, xvar and yvar are the only data needed 
# to create scatterplot
setMethod("initialize", "sPlotView", 
  function(.Object, ...)
  {
    dataName<-list(...)$dataName
    mData<-list(...)$mData

    dfRows<-mData$dfRows
    xvar<-mData$xvar
    yvar<-mData$yvar

    # create slot values
    retList<-initSPlotView()

    # fill the slots in the object
    .Object@dataName<-dataName
    .Object@win<-retList$win
    .Object@plotDevice<-retList$plotDevice
    .Object@plotPar<-retList$plotPar
    .Object@drArea<-retList$drArea
    .Object@dfRows<-dfRows
    .Object@xvar<-xvar
    .Object@yvar<-yvar

    # create the actual plot
    .Object<-scatterplot(.Object)

    # finally add events to the plot
    .Object<-addEventsforViews(.Object)
    addEventsforSPlots(.Object)

    .Object
  }
)

setMethod("initialize", "spreadView", 
  function(.Object, ...)
  {
    dataName<-list(...)$dataName
    mData<-list(...)$mData
    spreadData<-mData$spreadData
    colNames<-mData$colNames

    # create the slot values
    retList<-initSpreadView(dataName, spreadData, colNames)
   
    # fill the slots in the object
    .Object@dataName<-dataName
    .Object@win<-retList$win
    .Object@clist<-retList$clist

    # finally add events to the view
    .Object<-addEventsforViews(.Object)
    addEventsforSpread(.Object)

    .Object
  }
)

setMethod("initialize", "graphView",
  function(.Object, ...)
  {
    dataName<-list(...)$dataName
    mData<-list(...)$mData

    glayout<-mData$glayout
    nShape<-mData$nShape

    # create slot values
    retList<-initGraphView(dataName, glayout, nShape)

    # fill the slots in the object
    .Object@dataName<-dataName
    .Object@win<-retList$win
    .Object@plotDevice<-retList$plotDevice
    .Object@plotPar<-retList$plotPar
    .Object@drArea<-retList$drArea
    .Object@grLayout<-retList$graphLayout

#    # need to update the virtualData slot in the model
#    curMVC<-getMVC(dataName)
#    virtualData(model(curMVC))<-retList$graphLayout
#    # then need to update MVCList
#    MVCList<-get("MVCList", mvcEnv)
#    allNames<-getModelNames(sort=FALSE)
#    mvcindex<-match(dataName, allNames)    
#    MVCList[[mvcindex]]<-curMVC
#    assign("MVCList", MVCList, mvcEnv)

    # create the actual plot
    .Object<-graphPlot(.Object)

    # finally add events to the plot
    .Object<-addEventsforViews(.Object)
    addEventsforGraphs(.Object)

    .Object    
  }
)

setMethod("initialize", "heatmapView",
  function(.Object, ...)
  {
    dataName<-list(...)$dataName
    mData<-list(...)$mData

    # create slot values - can use the same method as used for initializing
    # a scatterplot view
    retList<-initSPlotView(allowResize=FALSE)

    # fill the slots in the object
    .Object@dataName<-dataName
    .Object@win<-retList$win
    .Object@plotDevice<-retList$plotDevice
    .Object@plotPar<-retList$plotPar
    .Object@drArea<-retList$drArea
    .Object@ordering<-list()
    if (length(mData) > 0)
      .Object@rNames<-mData$rNames
    else
    {
      curModelD<-modelData(model(getMVC(dataName)))
      if (is(curModelD, "exprSet"))
        .Object@rNames<-geneNames(curModelD)
      else
        .Object@rNames<-rownames(curModelD)
    }
    # create the actual plot
    .Object<-heatmapPlot(.Object)

    # finally add events to the plot
    .Object<-addEventsforViews(.Object)
    # 4/22/06 start by adding a click event for a heatmap
    addEventsforHeatmaps(.Object)

    .Object    
  }
)

############
# initialize methods for messages
############

# added 6/21/06
setMethod("initialize", "gAskAncestorMessage",
  function(.Object, ...)
  {
    .Object@type<-list(...)$type
    .Object@mData<-list(...)$mData
    fromName<-list(...)$from
    if (fromName == "")
      fromName<-get("activeMVC", mvcEnv)
    .Object@from<-fromName
    .Object
  }
)

# added 7/20/05
setMethod("initialize", "gSendChildMessage",
  function(.Object, ...)
  {
    .Object@parentUpdateDataMessage<-list(...)$parentUpdateDataMessage
    .Object@childName<-list(...)$childName
    .Object
  }
)

# added 7/14/05
setMethod("initialize", "gSendParentMessage",
  function(.Object, ...)
  {
    .Object@childUpdateDataMessage<-list(...)$childUpdateDataMessage
    .Object
  }
)

setMethod("initialize", "gUpdateViewMessage", 
  function(.Object, ...)
  {
    .Object@type<-list(...)$type
    .Object@mData<-list(...)$mData
    dataName<-list(...)$dataName
    if (dataName == "")
      dataName<-get("activeMVC", mvcEnv)
    .Object@dataName<-dataName

    .Object
  }
)

#####
# update the data stored in the modelData slot of a model
#####
setMethod("initialize", "gUpdateDataMessage", 
  function(.Object, ...)
  {
    .Object@type<-list(...)$type
    .Object@mData<-list(...)$mData
    dataName<-list(...)$dataName
    from<-list(...)$from
    if (dataName == "")
      dataName<-get("activeMVC", mvcEnv)
    if (from == "")
      from<-dataName
    .Object@dataName<-dataName
    .Object@from<-from

    .Object
  }
)

#######
# data must be a list!!!
# data may contain only the model data or it may contain the model data AND
# the link data
#######
setMethod("initialize", "gAddDataMessage",
  function(.Object, ...)
  {
    .Object@dataName<-list(...)$dataName
    .Object@mData<-list(...)$data
    .Object@type<-list(...)$type 

    .Object
  }
)

setMethod("initialize", "gAddViewMessage",
  function(.Object, ...)
  {
    .Object@dataName<-list(...)$dataName
    .Object@type<-list(...)$type
    .Object@mData<-list(...)$mData

    .Object
  }
)

# added 7/13/05 
setMethod("initialize", "gAddChildMessage",
  function(.Object, ...)
  {
    .Object@dataName<-list(...)$dataName
    .Object@mData<-list(...)$data
    .Object@type<-list(...)$type
  
    .Object
  }
)

##############
# handle message methods for messages
##############

# added 6/21/06
setMethod("handleMessage", "gAskAncestorMessage",
  function(object, ...)
  {
    # need to look for an ancestor and then call the provideInfo method
    fromName<-from(object)
    curData<-mData(object)
    curType<-type(object)

    # starting with the from MVC, look through its ancestors until the first
    # model of curType is found
    fromMVC<-getMVC(fromName)

    curMVC<-fromMVC
    parentMVCName<-parentMVC(curMVC)

    while (parentMVCName != "" && !is(model(curMVC), curType))
    {
      curMVC<-getMVC(parentMVCName)
      parentMVCName<-parentMVC(curMVC)
    }

    # now check if we found the right model type
    if (is(model(curMVC), curType))
    {
      pType<-names(curData)
      # then call the appropriate provideInfo method
      retValue<-provideInfo(model(curMVC), pType, curData)
      return(retValue)
    }
    else
      return(NULL)
  }
)

setMethod("handleMessage", "gSendChildMessage",
  function(object, ...)
  {
    curchildName<-childName(object)
   
    curMVC<-getMVC(curchildName)
    linkData<-linkData(model(curMVC))
    fromParent<-linkData$fromParent
    
    # send fromParent the gSendChildMessage object so fromParent will
    # get both the gUpdateDataMessage from the parent MVC and it will get
    # the current childName
    newUpdateMessage<-fromParent(object)
    # there may be no update message for the child if the child does
    # not contain the data that changed in the parent (i.e. the child is a
    # subset of the parent)
    if (!is.null(newUpdateMessage))
    {
      # set the from slot so the update data message knows that this message
      # came from the parent
      from(newUpdateMessage)<-dataName(parentUpdateDataMessage(object))
      handleMessage(newUpdateMessage)
    }
  }
)

setMethod("handleMessage", "gSendParentMessage",
  function(object, ...)
  {
    # need to get the toParent function, which is stored in the linkData
    # slot in the model object
    childName<-dataName(childUpdateDataMessage(object))
    curMVC<-getMVC(childName)

    linkData<-linkData(model(curMVC))
    # linkData is a list with 2 elements: toParent and fromParent
    toParent<-linkData$toParent

    # send toParent the gSendParentMessage object, which will contain
    # the gUpdateDataMessage for the child MVC
    # newUpdateMessage will be the gUpdateDataMessage for the parent
    # sometimes the toParent function returns NULL!!!
    newUpdateMessage<-toParent(object)

    # set the from slot so the update data message knows that this message
    # came from a child
    if (!is.null(newUpdateMessage))
    {
      from(newUpdateMessage)<-dataName(childUpdateDataMessage(object))
      handleMessage(newUpdateMessage)
    }
  }
)

setMethod("handleMessage", "gUpdateViewMessage",
  function(object, ...)
  {
    type<-type(object)
    viewdata<-mData(object)
    dataName<-dataName(object)

    # now need to update the view - note it may not be the active MVC
    curMVC<-getMVC(dataName)
    curVList<-viewList(curMVC)
    controlEnv<-controller(curMVC)

    if (length(dev.list()) > 0)
      activeDev<-dev.cur()

    assign("contLoop", TRUE, controlEnv)
    controller(curMVC)<-controlEnv
    MVCList<-get("MVCList", mvcEnv)
    allNames <- getModelNames(sort = FALSE)

    index<-match(dataName, allNames)
    MVCList[[index]]<-curMVC
    assign("MVCList", MVCList, mvcEnv)

    if (type=="updateView")
    {
      for (i in 1:length(curVList))
      {
        updateView(curVList[[i]], viewdata) 
      }
    }
    if (type=="redrawView")
    {
      for (i in 1:length(curVList))
      {
        # for type of "redrawView", mData will be an empty list
        redrawView(curVList[[i]])
      }
    }

    # need to get this again because the MVC object has changed
    curMVC<-getMVC(dataName)
    curVList<-viewList(curMVC)
    controlEnv<-controller(curMVC)

    assign("contLoop", FALSE, controlEnv)
    controller(curMVC)<-controlEnv
    MVCList<-get("MVCList", mvcEnv)
    allNames <- getModelNames(sort = FALSE)

    index<-match(dataName, allNames)
    MVCList[[index]]<-curMVC
    assign("MVCList", MVCList, mvcEnv)

    # reset the active device
    if (length(dev.list()) > 0)
      dev.set(activeDev)
  }
)

########
# added on 7/14/05 so that we also update parent/children of the model
########
setMethod("handleMessage", "gUpdateDataMessage",
  function(object,...)
  {
    data<-mData(object)
    type<-type(object)

    dataName<-dataName(object)
    from<-from(object)

    # now need to update the model
    curMVC<-getMVC(dataName)
    curModel<-model(curMVC)
    # will get an error here if print statement is used when the model to
    # be updated is a graph

    # a method for each model to update its data
    viewdata<-updateModel(curModel, type, data)

    # update the views now that the data has changed
    # check that the model has views also!
    curVList<-viewList(curMVC)
    if (length(curVList) > 0)
    {
      # create an update view message
      # for now assume that all update view messages that come from update
      # data will have type "updateView", rather than "redrawView"
      uvMessage<-new("gUpdateViewMessage", type="updateView", mData=viewdata,
                      dataName=dataName)
      handleMessage(uvMessage)
    }

    # after updating its views, now need to look for parent and children
    parentName<-parentMVC(curMVC)
    childrenName<-childMVCList(curMVC)

    # only send the message if there is a parent model and if that parent 
    # model is not where the update data message came from
    if (!(parentName %in% c("", from)))
    {
      sendToParent<-get("sendToParent", controller(curMVC))
      if (sendToParent==TRUE)
      {
        # parentMessage will be a new gUpdateDataMessage
        parentMessage<-new("gSendParentMessage", 
                           childUpdateDataMessage=object)
        handleMessage(parentMessage)
      }
    }
    if (length(childrenName) > 0)
    {
      sendToChild<-get("sendToChild", controller(curMVC))
      # can have multiple children (can only have one parent)
      for (i in 1:length(childrenName))
      {
        # only pass the message on to the children if there are children
        # and the child is not where the update data message came from
        if (!(childrenName[[i]] %in% c("", from)))
        {
          # childMessage will be a new gUpdateDataMessage
          childMessage<-new("gSendChildMessage", 
                            parentUpdateDataMessage=object, 
                            childName=childrenName[[i]])
          # need to know which child is being updated
          handleMessage(childMessage)
        }
      }
    }
  }
)

########
# 6/1/05
# load models through messages
########
setMethod("handleMessage", "gAddDataMessage",
  function(object, ...)
  {
    data<-mData(object)$data
    linkData<-mData(object)$linkData
    if (is.null(linkData))
      linkData<-list()
    name<-dataName(object)
    type<-type(object)

    loadModel(data, type, name, linkData)

    # need to refresh the model graph if there is one
    if (length(get("modelWin", mvcEnv)) > 0)
    {
      Sys.sleep(0.5)
      plotModelGraph()
    }
  }
)

#######
# 5/24/05 want to initialize a view through methods rather than a function
#######
setMethod("handleMessage", "gAddViewMessage",
  function(object, ...)
  {
    dataName<-dataName(object)
    type<-type(object)
    mData<-mData(object)

    if (length(mData) > 0)
      newView<-new(type, dataName=dataName, mData=mData)
    else
      newView<-new(type, dataName=dataName)
  }
)

##########
# 7/13/05 handle an add child message
# here ... will contain the 2 functions:
# toParent and fromParent, which will be
# used by the gSendParentMessage and gSendChildMessage
# it will also contain the subset of the parent data that was
# used to create this new model (variable called subsetParentData)
##########
setMethod("handleMessage", "gAddChildMessage",
  function(object, ...)
  {
    # will need to follow some of the same steps as the handle message method
    # for the gAddDataMessage class
    data<-mData(object)$data
    virtualData<-mData(object)$virtualData
    name<-dataName(object)
    type<-type(object)

    loadModel(data, type, name, virtualData=virtualData)

    # then need to set some extra slots in the parent and child MVCs
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    childMVC<-getMVC(name)
    
    # need to get the 2 functions that will go in linkData slot
    extraPar<-list(...)
    toParent<-extraPar$toParent
    fromParent<-extraPar$fromParent
    subsetParentData<-extraPar$subsetParentData

    # reassign the child MVC List in the parent MVC object
    curChildList<-childMVCList(curMVC)
    curChildList[length(curChildList)+1]<-name
    childMVCList(curMVC)<-curChildList

    # assign the parentMVC in the child MVC object
    parentMVC(childMVC)<-activeMVC

    linkData(model(childMVC))<-list(toParent=toParent, 
                                    fromParent=fromParent)
    assign("subsetParentData", subsetParentData, controller(childMVC))

    # now need to reassign these MVC objects to the MVCList
    MVCList<-get("MVCList", mvcEnv)
    allNames <- getModelNames(sort = FALSE)
    parentIndex <- match(activeMVC, allNames)
    MVCList[[parentIndex]]<-curMVC
    childIndex <- match(name, allNames)
    MVCList[[childIndex]]<-childMVC
    assign("MVCList", MVCList, mvcEnv)

    # need to refresh the model graph if there is one
    if (length(get("modelWin", mvcEnv)) > 0)
    {
      Sys.sleep(0.5)
      plotModelGraph()
    }
  }
)


