##########
# new functions for these new classes
##########

#######
# called by many functions in MVCClass and iSNetwork
#######
getMVC<-function(mName)
{
  mvcList<-get("MVCList", mvcEnv)
  allNames<-getModelNames(sort=FALSE)
  index<-match(mName, allNames)
  curMVC<-mvcList[[index]]
  return(curMVC)
}

#########
# called by many functions in MVCClass and iSNetwork
#########
getModelNames<-function(sort=TRUE)
{
  mvcList<-get("MVCList", mvcEnv)
  if (length(mvcList) > 0)
  {
    allNames<-unlist(lapply(lapply(mvcList, model), modelName))
    # return the names in alphabetical order
    if (sort)
      return(sort(allNames))
    else
      return(allNames)
  }
  else
    return(NULL)
}

#######
## should this be in MVCClass package or iSNetwork????
# load the data into the mvcEnv
# data is the name of data variable
# type determines the model class
# name is the character string name of the model
# linkData is the link data
#
# called by handleMessage method for gAddDataMessage
#######
loadModel<-function(data, type, name, linkData=list(), virtualData=NULL)
{
  booLoad<-TRUE
  # should check that the model name is not already in use!!!
  allMNames<-getModelNames()
  if (length(allMNames) > 0)
  {
    if (name %in% allMNames)
    {
      booLoad<-FALSE
      if (length(get("controlWin", mvcEnv)) > 0)
        showLabel("That model name is already in use.")
      else
        print("That model name is already in use.")
    }
  }

  if (booLoad)
  {
    # first create the appropriate model object
    newObject<-switch(type,
      "exprSet"=new("exprModel", modelData=data, modelName=name, 
                      linkData=linkData, virtualData=virtualData),
      "graph"=new("graphModel", modelData=data, modelName=name,
                      linkData=linkData, virtualData=virtualData),
      "data.frame"=new("dfModel", modelData=data, modelName=name, 
                      linkData=linkData, virtualData=virtualData),
      "GSE"=new("gseModel", modelData=data, modelName=name,
                      linkData=linkData, virtualData=virtualData)
    )

    # now need to create a MVC object and add to MVCList in mvcEnv
    # leave parentMVC slot as NULL
    newMVC<-new("linkedModelMVC", model=newObject, viewList=list(), 
                parentMVC="", controller=new.env(), childMVCList=list())

    # need to add the default values to the new controller environment
    setControllerDefaults(newMVC)
    createDefaultCallEvents(newMVC)
 
    MVCList<-get("MVCList", mvcEnv)
    if (length(MVCList) > 0)
    { 
      MVCList[[length(MVCList)+1]]<-newMVC
      assign("MVCList", MVCList, mvcEnv)
    }
    else
    {
      assign("MVCList", list(newMVC), mvcEnv)
      # if this is the first model, then it will be the active MVC
      assign("activeMVC", modelName(model(newMVC)), mvcEnv)
      if (length(get("controlWin", mvcEnv)) > 0)
      {
        # set the display menu if the control window is open
        setDisplayMenu(modelName(model(newMVC)))
      }
    }
  }
}

############
# show a label in a Gtk window
#
# called by loadModel function
############
showLabel<-function(lblText)
{
  win<-gtkWindow(show=FALSE)
  hbox<-gtkHBox()
  win$Add(hbox)
  lbl<-gtkLabel(lblText)
  gtkBoxPackStart(hbox,lbl,padding=5)
  okBut<-gtkButton("Ok")
  okBut$SetUsize(40,20)
  gtkBoxPackStart(hbox,okBut,padding=5)
  
  gtkAddCallback(okBut,"clicked",
    function(obj)
    {
      win$Destroy()
      return(TRUE)
    }
  )
  win$Show()
}

