\name{createGoGraph}
\alias{createGoGraph}
\title{ Creates a child model by creating a GO graph }
\description{
  createGoGraph is a function that creates a child model that is the gene
  ontology (GO) graph that corresponds to the affy (Affymetrix) ids from the
  parent model.  Thus, the active MVC must have a model of class exprModel
  (exprSet) or the active MVC must be a data frame model where the rows
  correspond to affy ids or a graph model where the nodes correspond to affy
  ids. 

  This function calls the makeGOGraph function to create the GO graph.
}
\usage{
createGoGraph(newModelName, annotationSlot, ontology = "MF", affyids = "",
              mVar = "")
}
\arguments{
  \item{newModelName}{ the name of the new child model }
  \item{annotationSlot}{ the annotation slot of the exprSet (the Affymetrix
  chip type, such as "hu6800", "hgu95av2", etc.) }
  \item{ontology}{ the type of ontology (can be "MF", "BP", or "CC") }
  \item{affyids}{ the Affymetrix ids from the parent model that are used to
  create the GO graph (if affyids is left empty, then all Affymetrix ids from
  the parent model are used) }
  \item{mVar}{ a model variable that is used to subset the parent model (mVar
  must be a model variable that is a logical vector); to see the model
  variables call getModelVar }
}
\details{
  Only use the affyids parameter if you want to create a GO graph from a
  subset of the Affymetrix ids in the active MVC.  If this parameter is left
  empty, all Affymetrix ids are used.  Similarly, users can subset the parent
  model using the mVar parameter.  If both affyids and mVar are set, then the
  Affymetrix ids that fit both criteria will be used to create the GO graph.
}
\author{ Elizabeth Whalen }
\seealso{ 
  \code{\link[GOstats]{makeGOGraph}}, 
  \code{\link{loadData}},
  \code{\link{getMethodsToCreateChild}},
  \code{\link{createSubset}},
  \code{\link{performMDS}},
  \code{\link{getModelVar}} 
}
\examples{
  if (interactive())
  {
    library(CLL)
    data(sCLLex)
    data(sFiltert)
    loadData(modelData=sCLLex, dataName="CLL", typeModel="exprSet")
    loadModelVar("CLL", "sFiltert", sFiltert)
    setActiveMVC("CLL")
    # now create the child model
    createGoGraph(newModelName="CLLgo", annotationSlot="hgu95av2", 
                  mVar="sFiltert")
    getModelNames()
  }
}
\keyword{ data }

