\name{createKEGGGSE}
\alias{createKEGGGSE}
\title{ Creates a child model by performing gene set enrichment (GSE) using
KEGG }
\description{
  createKEGGGSE is a function that creates a child model from an exprModel by
  performing gene set enrichment (GSE) using KEGG pathways as the gene sets.
  In this function, row t-tests are performed to obtain the gene versus
  phenotype test statistics and the gene set statistics, which are obtained by
  multiplying the incidence matrix by the gene set statistics, are
  standardized by dividing by the square root of the incidence matrix row sums
  (i.e. to approximate a N(0,1) distribution).
}
\usage{
createKEGGGSE(newModelName, fac, annot = "", affyids = "", mVar = "", noGenes = 5)
}
\arguments{
  \item{newModelName}{ the name of the new child model - it must be different
  from all other loaded model names (call getModelNames to see the loaded
  model names) }
  \item{fac}{ the phenotype factor to perform the row t-tests }
  \item{annot}{ the annotation of the exprModel (if left blank, the annotation
  slot of the exprSet is used) }
  \item{affyids}{ (optional) Affymetrix identifiers from the parent model that
  are used to create the new gseModel (i.e. this a way to use only a subset of
  the parent exprModel to create the child gseModel) }
  \item{mVar}{ (optional) the model variable (which must be a logical) to
  subset the exprModel Affymetrix identifiers, which will be used to create
  the new gseModel }
  \item{noGenes}{ the minimum number of genes a category (gene set) must
  contain to be included in the incidence matrix }
}
\details{
  The Category and genefilter packages are required for this function.
}
\author{ Elizabeth Whalen }
\seealso{ 
  \code{\link{createGoGraph}},
  \code{\link{createChrLocGSE}},
  \code{\link[BioMVCClass]{gseModel-class}}  
}
\examples{
  if (interactive())
  {
    library(CLL)
    data(sCLLex)
    data(nsFilter)
    loadData(modelData=sCLLex, dataName="CLL", typeModel="exprSet")
    loadModelVar("CLL", "nsFilter", nsFilter)
    setActiveMVC("CLL")    
    createKEGGGSE(newModelName="CLLgse", fac="Disease", mVar="nsFilter")
  }
}
\keyword{ data }

