\name{deleteMVC}
\alias{deleteMVC}
\title{ Delete a MVC object }
\description{
  deleteMVC will remove a MVC object, which means that the data set is removed
  and all of its views are closed.  deleteMVC performs the same functionality
  as the Delete MVC menu item under the Edit menu on the control window.  The
  active MVC object cannot be deleted.  Only non-active MVC objects can be
  deleted. 

  Also if the deleted MVC object is a parent MVC, then its children MVC
  objects will no longer have a parent MVC.  If the deleted MVC object is a
  child MVC, then the parent MVC object will no longer have this child MVC
  object.  Finally, if the deleted MVC object is a parent and a child MVC
  object, then deleting this MVC object will break the link between the
  deleted MVC's parent MVC and its children MVC objects (i.e. the grandparent
  MVC will no longer be linked to the grandchildren MVC objects).
}
\usage{
deleteMVC(dMVC)
}
\arguments{
  \item{dMVC}{ the name of the model (MVC) object to be removed }
}
\author{ Elizabeth Whalen }
\seealso{ 
  \code{\link{loadData}}, 
  \code{\link{getModelNames}},
  \code{\link{getActiveMVC}}
}
\examples{
  if (interactive())
  {
    loadData(modelData=USArrests, dataName="USArrests", typeModel="data.frame")
    setActiveMVC("USArrests")

    set.seed(123)
    V <- letters[1:10]
    M <- 1:4
    g1 <- randomGraph(V, M, .2)
    loadData(modelData=g1, dataName="graph1", typeModel="graph")    

    createSpreadsheet()
    setActiveMVC("graph1")
    createGraphPlot()
    # now delete the USArrests MVC (which is not active)
    deleteMVC("USArrests")
    getModelNames()
  }
}
\keyword{ data }

