\name{getHiddenValues}
\alias{getHiddenValues}
\title{ Return any data points that are hidden }
\description{
  getHiddenValues returns the data points in the model that are currently
  hidden (i.e. they have been purposedly hidden in views).  If the model in
  the active MVC is a dfModel object (data frame), then getHiddenValues will
  return any row names that are hidden; if the model is a graphModel object
  (graph), then getHiddenValues will return any node names that are hidden;
  and if the model is an exprModel object (exprSet), then getHiddenValues will
  return any gene names that are hidden.  If there are no hidden values in the
  model of the active MVC, then NULL is returned.
}
\usage{
getHiddenValues(printState = TRUE)
}
\arguments{
  \item{printState}{ a boolean to tell whether print statements should be written }
}
\value{
  Returns either a character string of the names of the hidden values or NULL
  if there are no hidden values.
}
\author{ Elizabeth Whalen }
\seealso{ 
  \code{\link{showHiddenValues}} 
}
\examples{
  if (interactive())
  {
    loadData(modelData=USArrests, dataName="USArrests",
       typeModel="data.frame")
    setActiveMVC("USArrests")
    createSPlot(varx="Murder", vary="Assault")
    setCallFunc("leftButtonClick", "hide a point")
    # now the user needs to click a point with the left button and then call:
    getHiddenValues()
  }
}
\keyword{ data }

