\name{getMethodsToCreateChild}
\alias{getMethodsToCreateChild}
\title{ Returns the functions to create a child model }
\description{
  getMethodsToCreateChild returns a vector of functions that can create
  child models for the active MVC.  This returned vector has names that
  correspond to the function names and values that describe what these
  functions do.  If you just want the names of the functions to create child
  models, then just look at the names for the returned vector.  
  
  Different active MVCs may have different functions to create a child model.
  For instance with a data frame model as the active data set, the functions
  to create a child model include creating a subset, performing MDS, and
  creating a GO graph.
}
\usage{
getMethodsToCreateChild()
}
\details{
  The getMethodsToCreateChild function returns a vector with the
  functions to create a child model.  For more information about the functions
  to create a child model, look at those functions' man pages.
}
\value{
  Returns a vector.  The names of the vector refer to names of functions that
  can create a child model for the active MVC.  The values of the vector
  describe what these functions do.
}
\author{ Elizabeth Whalen }
\seealso{ 
  \code{\link{createGoGraph}},
  \code{\link{createSubset}},
  \code{\link{performMDS}} 
}
\examples{
  if (interactive())
  {
    loadData(modelData=USArrests, dataName="USArrests",
        typeModel="data.frame")
    setActiveMVC("USArrests")
    getMethodsToCreateChild()
    # to just return the names of the functions
    names(getMethodsToCreateChild())
  }
}
\keyword{ data }

