\name{getNodeAttr}
\alias{getNodeAttr}
\title{ Get the Node Attributes for a Graph }
\description{
  If the active MVC contains a graph model, then getNodeAttr returns the names
  of the node attributes for the graph.  This function is useful when setting
  tooltips through the command line for a graph plot.  The tooltips for a
  graph plot can either be the node names or a node attribute.
}
\usage{
getNodeAttr()
}
\value{
  Returns a vector of the names of the node attributes.  If there are no node
  attributes, then NULL is returned.
}
\author{ Elizabeth Whalen }
\seealso{
  \code{\link{setTooltips}} 
}
\examples{
  if (interactive())
  {
    library(CLL)
    data(sCLLex)
    data(sFiltert)
    loadData(modelData=sCLLex, dataName="CLL", typeModel="exprSet")
    loadModelVar("CLL", "sFiltert", sFiltert)
    setActiveMVC("CLL")
    # now create the child model
    createGoGraph(newModelName="CLLgo", annotationSlot="hgu95av2", 
                  mVar="sFiltert")
    setActiveMVC("CLLgo")
    createGraphPlot()
    setCallFunc("mouseOver", "show tooltips over nodes")
    getNodeAttr()
    setTooltips("Term")
  }
}
\keyword{ data }
