\name{loadData}
\alias{loadData}
\title{ Loads a data set }
\description{
  The loadData function provides the same functionality as choosing the Load
  Model menu item from the File menu on the control window.  Currently, only
  three types of models can be loaded: data frame (or matrix), graph, and
  exprSet. 
}
\usage{
loadData(modelData, dataName, typeModel)
}
\arguments{
  \item{modelData}{ the actual data set }
  \item{dataName}{ the name for the data set (this is a character string) }
  \item{typeModel}{ the type of model; currently must be one of the following
  options: "data.frame", "graph", or "exprSet" }
}
\author{ Elizabeth Whalen }
\seealso{ 
  \code{\link{loadModel}}  
}
\examples{
  if (interactive())
  {
    # first load a data frame (the first data set loaded is the active data)
    loadData(modelData=USArrests, dataName="USArrests",
       typeModel="data.frame")
    # then load an exprSet
    library(CLL)
    data(sCLLex)
    loadData(sCLLex, "CLL", "exprSet")
    # then load a graph
    set.seed(123)
    V <- letters[1:10]
    M <- 1:4
    g1 <- randomGraph(V, M, .2)
    loadData(g1, "testGraph", "graph")
  }
}
\keyword{ data }

