\name{performMDS}
\alias{performMDS}
\title{ Create a child model by performing Multidimensional Scaling }
\description{
  performMDS is a function that creates a child model by performing
  Multidimensional Scaling (MDS) on the parent model.  Currently, this
  function can work with models of class dfModel (data frame or matrix) and
  exprModel (exprSet).  This function calls the cmdscale function to perform
  MDS. 
}
\usage{
performMDS(newModelName, k = 2, transpose = FALSE)
}
\arguments{
  \item{newModelName}{ the name of the new child model }
  \item{k}{ the number of dimensions returned (see the man page for cmdscale) }
  \item{transpose}{ a boolean to determine if the matrix should be transposed
  before performing MDS }
}
\details{
  The transpose parameter is not currently used because it makes creating the
  link data functions too complicated.  Right now when a child MVC is created
  using MDS, it is linked to its parent MVC by matching row names.  If the
  matrix was transposed, then this form of linking could not be done.  Thus,
  right now it is not possibly to transpose the matrix (the parameter is not
  used in the function definition, but it is there in case it is needed in the
  future). 
}
\author{ Elizabeth Whalen }
\seealso{ 
  \code{\link{cmdscale}},
  \code{\link{loadData}},
  \code{\link{getMethodsToCreateChild}},
  \code{\link{createSubset}},
  \code{\link{createGoGraph}} 
}
\examples{
  if (interactive())
  {
    loadData(modelData=USArrests, dataName="USArrests",
            typeModel="data.frame")
    setActiveMVC("USArrests")
    # now create the child model
    performMDS(newModelName="USAMDS")
    getModelNames()
  }
}
\keyword{ data }

