\name{removeKeyVals}
\alias{removeKeyVals}
\title{ Removes accelerator keys }
\description{
  removeKeyVals is a function that removes accelerator keys.  This function is
  called when a accelerator key is no longer needed, like whenever a menu item
  is removed from the menu on the control window.  This function is called
  when the active MVC changes because different MVCs have different possible
  views and thus, the Display menu changes and some accelerator keys (for the
  previous active MVC) must be removed.  

  To find the correct values for the parameters accelChar and action, the user
  should call getKeyVals.
}
\usage{
removeKeyVals(accelChar, action)
}
\arguments{
  \item{accelChar}{ the accelerator key (this is a character string of length
  one) }
  \item{action}{ the response that occurs when the key press event occurs
  (this may either be a function name or the text used in updateControlWindow)}
}
\details{
  The user probably does not need to call this function from the command line.
}
\author{ Elizabeth Whalen }
\seealso{ 
  \code{\link{addToKeyVals}},
  \code{\link{getKeyVals}} 
}
\examples{
  if (interactive())
  {
    loadData(modelData=USArrests, dataName="USArrrests",
             typeModel="data.frame")
    setActiveMVC("USArrests")
    x<-getKeyVals()
    # the length of x should be at least one
    length(x)

    createSpreadsheet()
    # if you type Ctrl+Q while the spreadsheet is active, a window will appear
    # asking if you'd like to quit
    
    # now let's remove this accelerator key
    # you need to put an underscore right before the accelerator key in the 
    # text parameter
    removeKeyVals(accelChar="Q", action="quit")

    y<-getKeyVals()
    # the length of y will be the length of x because an accelerator key has
    # been added for the spreadsheet window and the accelerator key for
    # quitting has been removed so they cancel each other out
    length(y)

    # now type Ctrl+Q while the spreadsheet view is active - this time no
    # window will appear asking if you'd like to quit   
  }
}
\keyword{ data }

