\name{setModelName}
\alias{setModelName}
\title{ Used to set the name of a child model }
\description{
  When a user creates a heatmap by interacting with a node from a graph plot,
  a new child model is created along with the heatmap.  This new child model
  must have a name that is unique from all the other loaded model names.  If
  the user performs this interaction while using the GUI, then an interface
  will appear each time the user interacts with a node asking for the name of
  the new model.  However, if the user is performing this interaction (to
  create a heatmap from a graph node) while using the command line, the user
  must set the model name before each interaction with a node. 
}
\usage{
setModelName(childName, parentName = "")
}
\arguments{
  \item{childName}{ the name of the new child model - remember it must be
  unique from all the other model names }
  \item{parentName}{ the default is to assume that the parent of the new child
  will be the active MVC (model); if this is not true, then the user must
  supply the name of the parent (i.e. the name of the graph model) }
}
\details{
  To find the names of all the loaded models, call getModelNames.  Any name
  that is not returned from getModelNames can be used as the name of the new
  child model.
}
\author{ Elizabeth Whalen }
\seealso{  
  \code{\link{getModelNames}},
  \code{\link{setCallFunc}} 
}
\examples{
  if (interactive())
  {
    library(CLL)
    data(sCLLex)
    data(sFiltert)
    class(sFiltert)
    loadData(modelData=sCLLex, dataName="CLL", typeModel="exprSet")
    setActiveMVC("CLL")
    loadModelVar(modelName="CLL", modelVarName="sFiltert", modelVar=sFiltert)
    createHeatmap(mVar="sFiltert")
    createGoGraph(newModelName="CLLgo", annotationSlot="hgu95av2", 
                  mVar="sFiltert")
    setActiveMVC("CLLgo")
    createGraphPlot()
    setCallFunc("leftButtonClick", "create a heatmap")
    setModelName("CLLsub")
    # now click a node on the graph plot and a heatmap will be created
    # this new heatmap will be a plot of the new model, CLLsub
  }
}
\keyword{ data }

