\name{setTooltips}
\alias{setTooltips}
\title{ Sets the tooltips }
\description{
  setTooltips sets the tooltip values when the callback function is to show
  tooltips over objects in the plot (for instance, over points or over nodes).
  The default value for tooltips is to use the row names if the model is of
  class 'dfModel', to use node names if the model is of class
  'graphModel', and to use gene set names if the model is of class 'gseModel'.
  However, the user can also set the tooltips to be values from
  a variable stored with the model (such as values from a column in
  the data frame). 
}
\usage{
setTooltips(name)
}
\arguments{
  \item{name}{ can either be the name of a variable (such as a column name)
  from the active model or it can be 'rowNames' (if the model contains a data
  frame), or 'nodeNames' (if the model contains a graph), or 'geneSetNames' or
  'geneSetTestStats' (if the model contains a GSE - gene set enrichment) }
}
\details{
  For a data frame model, if the user wants the tooltips to be values from a
  column, then the parameter, name, should be the name of that column.  If the
  user wants the tooltips to be the row names, then the parameter, name,
  should be 'rowNames'.

  For a graph model, currently the options are to show node names or a model
  variable in the tooltips so the name parameter can be 'nodeNames' or the
  name of a model variable.

  For a GSE (gene set enrichment) model, the options are to show gene set
  names, gene set test statistics, or a model variable so the name parameter
  can be 'geneSetNames', 'geneSetTestStats', or the name of a model variable.
}
\author{ Elizabeth Whalen }
\seealso{ 
  \code{\link{setCallFunc}},
  \code{\link{getTooltips}},
  \code{\link{getNodeAttr}}
}
\examples{
  if (interactive())
  {
    loadData(modelData=USArrests, dataName="USArrests",
             typeModel="data.frame")
    setActiveMVC("USArrests")
    createSPlot(varx="Murder", vary="Assault")
    getDescForEvent("mouseOver")
    setCallFunc("mouseOver", "show tooltips over points")
    setTooltips("Murder")
    # now move the cursor over the plot
    Sys.sleep(3)
    setTooltips("rowNames")
  }
}
\keyword{ data }

